'****************************************************************
'*  Name    : Eight7SegDisplay.BAS                              *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2018 [No Copyright]                 *
'*          : All Rights Reserved                               *
'*  Date    : 22/06/2018                                        *
'*  Version : 1.0                                               *
'*  Notes   : Proton Basic Compiler V3.6.0.3                    *
'*          : Original code written by Les Johnson              *
'* Library  : MultiplexA8Digits_Lib.Inc                         *
'****************************************************************
'/ Code for 8 seven segment display with logic interface for digit value
'/ and digit enable to reduce pins count.
'/ Using the 74HC4511 decoder, a table of segments value is not needed.
'/ Using the 74HC238 3 to 8 lines reduces the pins count.
'/ Total pins used for 8 digits = 8 bits only.
'/
'/ 28/06/2018 version 3.0
'/ Add a macro that allows to input every byte of your main program.
'/ The type of the variable is checked and makes an error at compiling if 
'/ the type of variable is diffrent as library.
'/
    Device = 18F452
   
    Declare Xtal = 8    ' For Proteus testing
'    Declare Xtal = 40   ' For real PIC (40Mhz or 64Mhz depending the family)

    Declare Optimiser_Level = 3
    Declare Dead_Code_Remove = On
    Declare Show_System_Variables = On
    Declare Create_Coff = On
    Declare Watchdog = OFF
    Declare Bootloader = OFF
    Declare Eeprom_Address $F00000  ' This address changes with K40/K42 PICs.
    '/ Disable this line for a real piece.
    $define _ProteusTest

'/ Configuration of the Serial comunication
    Declare Hserial_Baud = 9600                 ' Set baud rate to 9600
    Declare Hserial_RCSTA = %10010000           ' Enable serial port and continuous receive
    Declare Hserial_TXSTA = %00100100           ' Enable transmit and asynchronous mode
    Declare Hserial_Clear = On                  ' Enable Error clearing on received characters

'/ Rsout / Serial Constants
    Symbol Home = 1
    Symbol Bell = 7
    Symbol BkSp = 8
    Symbol Tab = 9
    Symbol CR = 13
    Symbol LF = 10    

'/ Temporal variables
    Dim TempByte As Byte
    Dim TestLoop As Word
    Dim TestNumbers As Byte
    Dim MyByte As Byte
    Dim MyWord As Word
    Dim MyDword As Dword 
'===============================================================================
'    Include "D:\MyIncFiles\MultiplexA8DigitsDecoder_Lib.inc"
    Include "MultiplexA8DigitsDecoder_Lib.inc"
'===============================================================================
MAIN_CODE:
    DelayMS 200
    $if _Hardware = 1
    HRSOut "DISPLAY RUNNING IN COMMON CATHODE MODE WITH THE 74HC4511 DECODER", CR,LF    
    HRSOut "THE 74HC238 DECODER & ULN2803 ENABLE THE DIGITS", CR,LF  
    $else
        $if DisplayMode = 0
        HRSOut "DISPLAY RUNNING IN COMMON CATHODE MODE WITH NPN TRANSITORS", CR,LF    
        $else
        HRSOut "DISPLAY RUNNING IN COMMON ANODE MODE WITH PNP TRANSITORS", CR,LF     
        $endif    
    $endif

    $if NumberDigits = 8
    HRSOut "DISPLAY RUNNING WITH 8 DIGITS", CR,LF     
    $endif
    $if NumberDigits = 7
    HRSOut "DISPLAY RUNNING WITH 7 DIGITS", CR,LF     
    $endif
    $if NumberDigits = 6
    HRSOut "DISPLAY RUNNING WITH 6 DIGITS", CR,LF     
    $endif
    $if NumberDigits = 5
    HRSOut "DISPLAY RUNNING WITH 5 DIGITS", CR,LF     
    $endif
    $if NumberDigits = 4
    HRSOut "DISPLAY RUNNING WITH 4 DIGITS", CR,LF     
    $endif
    $if NumberDigits = 3
    HRSOut "DISPLAY RUNNING WITH 3 DIGITS", CR,LF     
    $endif
    $if NumberDigits = 2
    HRSOut "DISPLAY RUNNING WITH 2 DIGITS", CR,LF     
    $endif

    HRSOut "DISPLAY INITIALIZED & TEST IS RUNNING", CR,LF

'===============================================================================
Multiplex:
    Do
        MyDword = 0
        BuildDigits(MyDword)			        '/ Extract the individual digits of DISPVAL
        DelayMS 500

        '/ First write 0 to 9.
       	For MyDword = 0 To 9 Step 1
        	BuildDigits(MyDword)			        '/ Extract the individual digits of DISPVAL
    		DelayMS 175
        Next

'/------------------------------------------------------------------------------
$if NumberDigits = 8
        $if _HEXnumbers = YES
        '/ Try to print HEX values
        WREG = ERead $0A
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        THOUSANDTHOUSANDS = WREG
        TENTHDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0B
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        THOUSANDTHOUSANDS = WREG
        TENTHDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0C
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        THOUSANDTHOUSANDS = WREG
        TENTHDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750        

        WREG = ERead $0D
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        THOUSANDTHOUSANDS = WREG
        TENTHDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0E
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        THOUSANDTHOUSANDS = WREG
        TENTHDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0F
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        THOUSANDTHOUSANDS = WREG
        TENTHDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 1000
        $endif

        $if ShiftingDigits = YES
        MyDword = 12345678
        BuildDigits(MyDword)			    '/ Extract the individual digits of DISPVAL
        DelayMS 500
        RotateRightDisplay(8,500)
        DelayMS 500

        RotateLeftDisplay(8,500)
        DelayMS 1000
        $endif

        TestNumbers = 0
        TestLoop = 0
        MyDword = 99999
        Repeat
            HRSOut Dec MyDword,","
            BuildDigits(MyDword)			    '/ Extract the individual digits of DISPVAL
            DelayMS 150
            MyDword = MyDword + 500001
            Inc TestNumbers
            If TestNumbers = 20 Then
                TestNumbers = 0
                HRSOut CR,LF
            EndIf
            Inc TestLoop 
        Until TestLoop > 198
        BuildDigits(MyDword)
        HRSOut Dec MyDword
        HRSOut CR,LF,CR,LF
        DelayMS 1000
$endif
'/------------------------------------------------------------------------------
$if NumberDigits = 7
        $if ShiftingDigits = YES
        MyDword = 1234567
        BuildDigits(MyDword)			    '/ Extract the individual digits of DISPVAL
        DelayMS 500
        RotateRightDisplay(7,500)
        DelayMS 500

        RotateLeftDisplay(7,500)
        DelayMS 1000
        $endif

        TestNumbers = 0
        TestLoop = 0
        MyDword = 899000
        Repeat
            HRSOut Dec MyDword,","
            BuildDigits(MyDword)			    '/ Extract the individual digits of DISPVAL
            DelayMS 150
            MyDword = MyDword + 100001
            Inc TestNumbers
            If TestNumbers = 20 Then
                TestNumbers = 0
                HRSOut CR,LF
            EndIf
            Inc TestLoop 
        Until TestLoop > 90
        BuildDigits(MyDword)
        HRSOut Dec MyDword
        HRSOut CR,LF,CR,LF
        DelayMS 1000
$endif

$if NumberDigits = 6
        $if ShiftingDigits = YES
        MyDword = 123456
        BuildDigits(MyDword)			    '/ Extract the individual digits of DISPVAL
        DelayMS 500
        RotateRightDisplay(6,500)
        DelayMS 500

        RotateLeftDisplay(6,500)
        DelayMS 1000
        $endif

        TestNumbers = 0
        TestLoop = 0
        MyDword = 99000
        Repeat
            HRSOut Dec MyDword,","
            BuildDigits(MyDword)			    '/ Extract the individual digits of DISPVAL
            DelayMS 150
            MyDword = MyDword + 11001
            Inc TestNumbers
            If TestNumbers = 20 Then
                TestNumbers = 0
                HRSOut CR,LF
            EndIf
            Inc TestLoop 
        Until TestLoop > 80
        BuildDigits(MyDword)
        HRSOut Dec MyDword
        HRSOut CR,LF,CR,LF
        DelayMS 1000
$endif
    
$if NumberDigits = 5
        $if ShiftingDigits = YES
        MyDword = 12345
        BuildDigits(MyDword)			    '/ Extract the individual digits of DISPVAL
        DelayMS 500
        RotateRightDisplay(5,500)
        DelayMS 500

        RotateLeftDisplay(5,500)
        DelayMS 1000
        $endif

        TestNumbers = 0
        TestLoop = 0
        MyDword = 9900
        Repeat
            HRSOut Dec MyDword,","
            BuildDigits(MyDword)			    '/ Extract the individual digits of DISPVAL
            DelayMS 150
            MyDword = MyDword + 1101
            Inc TestNumbers
            If TestNumbers = 20 Then
                TestNumbers = 0
                HRSOut CR,LF
            EndIf
            Inc TestLoop 
        Until TestLoop > 80
        BuildDigits(MyDword)
        HRSOut Dec MyDword
        HRSOut CR,LF,CR,LF
        DelayMS 1000
$endif

$if NumberDigits = 4
        $if ShiftingDigits = YES
        MyWord = 1234
        BuildDigits(MyWord)			    '/ Extract the individual digits of DISPVAL
        DelayMS 500
        RotateRightDisplay(4,500)
        DelayMS 500

        RotateLeftDisplay(4,500)
        DelayMS 1000
        $endif

        TestNumbers = 0
        TestLoop = 0
        MyWord = 900
        Repeat
            HRSOut Dec MyWord,","
            BuildDigits(MyWord)			    '/ Extract the individual digits of DISPVAL
            DelayMS 150
            MyWord = MyWord + 101
            Inc TestNumbers
            If TestNumbers = 25 Then
                TestNumbers = 0
                HRSOut CR,LF
            EndIf
            Inc TestLoop 
        Until TestLoop > 89
        BuildDigits(MyWord)
        HRSOut Dec MyWord
        HRSOut CR,LF,CR,LF
        DelayMS 1000
$endif
        
$if NumberDigits = 3
        $if ShiftingDigits = YES
        MyWord = 123
        BuildDigits(MyWord)			    '/ Extract the individual digits of DISPVAL
        DelayMS 500
        RotateRightDisplay(3,500)
        DelayMS 500

        RotateLeftDisplay(3,500)
        DelayMS 1000
        $endif

        TestNumbers = 0
        TestLoop = 0
        MyWord = 99
        Repeat
            HRSOut Dec MyWord,","
            BuildDigits(MyWord)			    '/ Extract the individual digits of DISPVAL
            DelayMS 150
            MyWord = MyWord + 14
            Inc TestNumbers
            If TestNumbers = 25 Then
                TestNumbers = 0
                HRSOut CR,LF
            EndIf
            Inc TestLoop 
        Until TestLoop > 63
        BuildDigits(MyWord)
        HRSOut Dec MyWord
        HRSOut CR,LF,CR,LF
        DelayMS 1000
$endif
                
$if NumberDigits = 2
       	For MyWord = 0 To 99 Step 1
        	BuildDigits(MyWord)			'/ Extract the individual digits of DISPVAL
    		DelayMS 175
        Next
$endif
'/------------------------------------------------------------------------------
        DelayMS 1000
    Loop      
'===============================================================================        
    End
'===============================================================================
        
