'****************************************************************
'*  Name    : Eight7SegDisplay_Neg.BAS                          *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2018 [No Copyright]                 *
'*          : All Rights Reserved                               *
'*  Date    : 22/06/2018                                        *
'*  Version : 3.0                                               *
'*  Notes   : Proton Basic Compiler V3.6.0.3                    *
'*          : Original code written by Les Johnson              *
'* Library  : MultiplexB8DigitsNeg_Lib.Inc                      *
'****************************************************************
'/ 15/06/2018-Version 2.0
'/ In the [BuildDigits()] routine, for the DP, it is necessary to clear one bit only.
'/ Then 3 lines of code could be saved.
'/ One digit added. Now the code can work with 2 to 6 digits.
'/ Using Buffers for ONES, TENS, HUNDREDS, THOUSANDS, etc... some blinking could be avoided.
'/ Better efficiency of the routines (faster).
'/
'/ 19/06/2018-Version 2.1
'/ Added the possibility of printing HEX numbers.
'/ Code for 8 digits display.
'/ Shifting the display to left and right.
'/
'/ 22/06/2018-Version 3.1
'/
'/ 28/06/2018 version 2.0
'/ Add a macro that allows to input every byte of your main program.
'/ The type of the variable is checked and makes an error at compiling if 
'/ the type of variable is diffrent as library.
'/
    Device = 18F452
    Declare Xtal = 8    ' For Proteus testing
'    Declare Xtal = 40   ' For real PIC (40Mhz, 48Mhz or 64Mhz depending the PIC series)

    Declare Optimiser_Level = 3
    Declare Dead_Code_Remove = On
    Declare Show_System_Variables = On
    Declare Create_Coff = On
    Declare Watchdog = OFF
    Declare Bootloader = OFF
    Declare Eeprom_Address $F00000  ' This address changes with K40/K42 PICs.
    '/ Disable this line for a real piece.
    $define _ProteusTest
        
'/ Configuration of the Serial comunication
    Declare Hserial_Baud = 9600                 ' Set baud rate to 9600
    Declare Hserial_RCSTA = %10010000           ' Enable serial port and continuous receive
    Declare Hserial_TXSTA = %00100100           ' Enable transmit and asynchronous mode
    Declare Hserial_Clear = On                  ' Enable Error clearing on received characters

'/ Rsout / Serial Constants
    Symbol Home = 1
    Symbol Bell = 7
    Symbol BkSp = 8
    Symbol Tab = 9
    Symbol CR = 13
    Symbol LF = 10    

'/ Temporal variables
    Dim TempByte As Byte
    Dim TestLoop As Word
    Dim TestNumbers As Byte
    Dim MySByte As SByte
    Dim MySWord As SWord
    Dim MySDword As SDword 
'===============================================================================
'    Include "D:\MyIncFiles\MultiplexB8DigitsNeg_Lib.inc"
    Include "MultiplexB8DigitsNeg_Lib.inc"
'===============================================================================
MAIN_CODE:
    DelayMS 200

    $if _Hardware = 0
        $if DisplayMode = 0
        HRSOut "DISPLAY RUNNING IN COMMON CATHODE MODE WITH NPN TRANSITORS", CR,LF    
        $else
        HRSOut "DISPLAY RUNNING IN COMMON ANODE MODE WITH PNP TRANSITORS", CR,LF     
        $endif    
    $else
        $error "ERROR in hardware. 2 PORTS configuration only"
    $endif
    
    $if NumberDigits = 8
    HRSOut "DISPLAY RUNNING WITH 8 DIGITS", CR,LF     
    $endif
    $if NumberDigits = 7
    HRSOut "DISPLAY RUNNING WITH 7 DIGITS", CR,LF     
    $endif
    $if NumberDigits = 6
    HRSOut "DISPLAY RUNNING WITH 6 DIGITS", CR,LF     
    $endif
    $if NumberDigits = 5
    HRSOut "DISPLAY RUNNING WITH 5 DIGITS", CR,LF     
    $endif
    $if NumberDigits = 4
    HRSOut "DISPLAY RUNNING WITH 4 DIGITS", CR,LF     
    $endif
    $if NumberDigits = 3
    HRSOut "DISPLAY RUNNING WITH 3 DIGITS", CR,LF     
    $endif
    $if NumberDigits = 2
    HRSOut "DISPLAY RUNNING WITH 2 DIGITS", CR,LF     
    $endif
    HRSOut "DISPLAY INITIALIZED & TEST IS RUNNING", CR,LF

'===============================================================================
Multiplex:
    Do
        '/ First write 0 to 9.
       	For MySDword = 0 To 9 Step 1
        	BuildDigits(MySDword)			   '/ Extract the individual digits of DISPVAL
    		DelayMS 175
        Next
'/------------------------------------------------------------------------------
$if NumberDigits = 8
        '/ Try to print HEX values
        WREG = ERead $0A
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        THOUSANDTHOUSANDS = WREG
        TENTHDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0B
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        THOUSANDTHOUSANDS = WREG
        TENTHDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0C
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        THOUSANDTHOUSANDS = WREG
        TENTHDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750        

        WREG = ERead $0D
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        THOUSANDTHOUSANDS = WREG
        TENTHDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0E
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        THOUSANDTHOUSANDS = WREG
        TENTHDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0F
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        THOUSANDTHOUSANDS = WREG
        TENTHDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 1000

        $if ShiftingDigits = YES
        MySDword = -1234567
        BuildDigits(MySDword)			    '/ Extract the individual digits of DISPVAL
        DelayMS 500
        RotateRightDisplay(8,500)
        DelayMS 500

        MySDword = -1234567
        BuildDigits(MySDword)			    '/ Extract the individual digits of DISPVAL
        DelayMS 500
        RotateLeftDisplay(8,500)
        DelayMS 1000
        $endif

        TestNumbers = 0
        TestLoop = 0
        MySDword = -9999999
        Repeat
            HRSOut SDec MySDword,","
            BuildDigits(MySDword)			    '/ Extract the individual digits of DISPVAL
            DelayMS 150
            MySDword = MySDword + 100000
            Inc TestNumbers
            If TestNumbers = 20 Then
                TestNumbers = 0
                HRSOut CR,LF
            EndIf
            Inc TestLoop 
        Until TestLoop > 199
        BuildDigits(MySDword)
        HRSOut SDec MySDword
        HRSOut CR,LF,CR,LF
        DelayMS 2000
$endif
'/------------------------------------------------------------------------------
$if NumberDigits = 7
        '/ Try to print HEX values
        WREG = ERead $0A
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        THOUSANDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0B
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        THOUSANDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0C
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        THOUSANDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750        

        WREG = ERead $0D
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        THOUSANDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0E
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        THOUSANDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0F
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        THOUSANDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 1000


        MySDword = -999999
        BuildDigits(MySDword)			    '/ Extract the individual digits of DISPVAL
        DelayMS 1000

        MySDword = -99999
        BuildDigits(MySDword)			    '/ Extract the individual digits of DISPVAL
        DelayMS 1000

        MySDword = -9999
        BuildDigits(MySDword)			    '/ Extract the individual digits of DISPVAL
        DelayMS 1000

        MySDword = -999
        BuildDigits(MySDword)			    '/ Extract the individual digits of DISPVAL
        DelayMS 1000
        
        MySDword = -99
        BuildDigits(MySDword)			    '/ Extract the individual digits of DISPVAL
        DelayMS 1000        

        MySDword = -9
        BuildDigits(MySDword)			    '/ Extract the individual digits of DISPVAL
        DelayMS 1000

        $if ShiftingDigits = YES
        MySDword = -123456
        BuildDigits(MySDword)			    '/ Extract the individual digits of DISPVAL
        DelayMS 500
        RotateRightDisplay(7,500)
        DelayMS 500

        MySDword = -123456
        BuildDigits(MySDword)			    '/ Extract the individual digits of DISPVAL
        DelayMS 500
        RotateLeftDisplay(7,500)
        DelayMS 1000
        $endif

        TestNumbers = 0
        TestLoop = 0
        MySDword = -999999
        Repeat
            HRSOut SDec MySDword,","
            BuildDigits(MySDword)			    '/ Extract the individual digits of DISPVAL
            DelayMS 150
            MySDword = MySDword + 10000
            Inc TestNumbers
            If TestNumbers = 20 Then
                TestNumbers = 0
                HRSOut CR,LF
            EndIf
            Inc TestLoop 
        Until TestLoop > 199
        BuildDigits(MySDword)
        HRSOut SDec MySDword
        HRSOut CR,LF,CR,LF
        DelayMS 2000
$endif
'/------------------------------------------------------------------------------
$if NumberDigits = 6
        '/ Try to print HEX values
        WREG = ERead $0A
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0B
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0C
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750        

        WREG = ERead $0D
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0E
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0F
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        HUNDREDTHOUSANDS = WREG
        PrintDigits()
        DelayMS 1000

        $if ShiftingDigits = YES
        MySDword = -12345
        BuildDigits(MySDword)			    '/ Extract the individual digits of DISPVAL
        DelayMS 500
        RotateRightDisplay(6,500)
        DelayMS 500

        MySDword = -12345
        BuildDigits(MySDword)			    '/ Extract the individual digits of DISPVAL
        DelayMS 500
        RotateLeftDisplay(6,500)
        DelayMS 1000
        $endif

        TestNumbers = 0
        TestLoop = 0
        MySDword = -99999
        Repeat
            HRSOut SDec MySDword,","
            BuildDigits(MySDword)			    '/ Extract the individual digits of DISPVAL
            DelayMS 150
            MySDword = MySDword + 1000
            Inc TestNumbers
            If TestNumbers = 20 Then
                TestNumbers = 0
                HRSOut CR,LF
            EndIf
            Inc TestLoop 
        Until TestLoop > 199
        BuildDigits(MySDword)
        HRSOut SDec MySDword
        HRSOut CR,LF,CR,LF
        DelayMS 2000
        
$endif
'/------------------------------------------------------------------------------
$if NumberDigits = 5
        '/ Try to print HEX values
        WREG = ERead $0A
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0B
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0C
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750        

        WREG = ERead $0D
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0E
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0F
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        TENTHOUSANDS = WREG
        PrintDigits()
        DelayMS 1000

        $if ShiftingDigits = YES
        MySDword = -1234
        BuildDigits(MySDword)			    '/ Extract the individual digits of DISPVAL
        DelayMS 500
        RotateRightDisplay(5,500)
        DelayMS 500

        MySDword = -1234
        BuildDigits(MySDword)			    '/ Extract the individual digits of DISPVAL
        DelayMS 500
        RotateLeftDisplay(5,500)
        DelayMS 1000
        $endif

        TestLoop = 0
        TestNumbers = 0
        MySDword = -9999
        Repeat
            HRSOut SDec MySDword,","
            BuildDigits(MySDword)			    '/ Extract the individual digits of DISPVAL
            DelayMS 150
       	    MySDword = MySDword + 200
            Inc TestNumbers
            If TestNumbers = 20 Then
                TestNumbers = 0
                HRSOut CR,LF
            EndIf
            Inc TestLoop
        Until TestLoop > 99
        BuildDigits(MySDword)
        HRSOut SDec MySDword
        HRSOut CR,LF,CR,LF
$endif
'/------------------------------------------------------------------------------
$if NumberDigits = 4
        '/ Try to print HEX values
        WREG = ERead $0A
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0B
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0C
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        PrintDigits()
        DelayMS 750        

        WREG = ERead $0D
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0E
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0F
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        THOUSANDS = WREG
        PrintDigits()
        DelayMS 1000

        $if ShiftingDigits = YES
        MySWord = -123
        BuildDigits(MySWord)			    '/ Extract the individual digits of DISPVAL
        DelayMS 500
        RotateRightDisplay(4,500)
        DelayMS 500

        MySWord = -123
        BuildDigits(MySWord)			    '/ Extract the individual digits of DISPVAL
        DelayMS 500
        RotateLeftDisplay(4,500)
        DelayMS 1000
        $endif


        TestLoop = 0
        TestNumbers = 0
        MySWord = -999
        Repeat
            HRSOut SDec MySWord,","
            BuildDigits(MySWord)			    '/ Extract the individual digits of DISPVAL
            DelayMS 250
       	    MySWord = MySWord + 10
            Inc TestNumbers
            If TestNumbers = 20 Then
                TestNumbers = 0
                HRSOut CR,LF
            EndIf
            Inc TestLoop
        Until TestLoop > 199
        BuildDigits(MySWord)
        HRSOut SDec MySWord
        HRSOut CR,LF,CR,LF

$endif
'/------------------------------------------------------------------------------        
$if NumberDigits = 3
        '/ Try to print HEX values
        WREG = ERead $0A
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0B
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0C
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        PrintDigits()
        DelayMS 750        

        WREG = ERead $0D
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0E
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0F
        ONES = WREG
        TENS = WREG
        HUNDREDS = WREG
        PrintDigits()
        DelayMS 1000

        TestLoop = 0
        TestNumbers = 0
        MySByte = -99
        Repeat
            HRSOut SDec MySByte,","
            BuildDigits(MySByte)			    '/ Extract the individual digits of DISPVAL
            DelayMS 300
       	    MySByte = MySByte + 1
            Inc TestNumbers
            If TestNumbers = 30 Then
                TestNumbers = 0
                HRSOut CR,LF
            EndIf
            Inc TestLoop
        Until TestLoop > 198
        BuildDigits(MySByte)
        HRSOut SDec MySByte
        HRSOut CR,LF,CR,LF
$endif
'/------------------------------------------------------------------------------
$if NumberDigits = 2
        '/ Try to print HEX values
        WREG = ERead $0A
        ONES = WREG
        TENS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0B
        ONES = WREG
        TENS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0C
        ONES = WREG
        TENS = WREG
        PrintDigits()
        DelayMS 750        

        WREG = ERead $0D
        ONES = WREG
        TENS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0E
        ONES = WREG
        TENS = WREG
        PrintDigits()
        DelayMS 750

        WREG = ERead $0F
        ONES = WREG
        TENS = WREG
        PrintDigits()
        DelayMS 1000
        
        TestLoop = 0
        TestNumbers = 0
        MySByte = -9
        Repeat
            HRSOut SDec MySByte,","
            BuildDigits(MySByte)			    '/ Extract the individual digits of DISPVAL
            DelayMS 200
       	    MySByte = MySByte + 1
            Inc TestNumbers
            If TestNumbers = 20 Then
                TestNumbers = 0
                HRSOut CR,LF
            EndIf
            Inc TestLoop
        Until TestLoop > 18     '17
        BuildDigits(MySByte)
        HRSOut SDec MySByte
        HRSOut CR,LF,CR,LF
        $endif
'/------------------------------------------------------------------------------
        DelayMS 2000
    Loop      
'===============================================================================        
    End
'===============================================================================
        
