'****************************************************************
'*  Name    : MulitplexA8DigitsDecoder_Lib.inc                  *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2018 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 22/06/2018                                        *
'*  Version : 1.0                                               *
'*  Notes   : Proton Basic Compiler V3.6.0.3                    *
'*          : Original Code written by Les Johnson              *
'* Purpose  : Multiplexing 8 digits LED display with interfaces *
'* Hardware : PIC, 74HC238, 74HC5411, ULN2003, 8 digits display *
'****************************************************************
'/ For other devices.
'/ Check and modify all code.
'/
'/ 28/06/2018 version 2.0
'/ Add a macro that allows to input every byte of your main program.
'/ The type of the variable is checked and makes an error at compiling if 
'/ the type of variable is diffrent as library.
'/

$ifndef _MultiplexDigits
$define _MultiplexDigits

'/------------------------------------------------------------------------------
    $if _device != _18F452
    $error "This library is for the PIC18F452 device only. Please write a new library"
    $endif

    $ifdef _ProteusTest
        $if _xtal != 8
        $error "It is better to use the Declare Xtal = 8 for Proteus"
        $endif   
    $endif
'===============================================================================
'/ Configuration of the Config Fuses
    $define PLL_ConfigFuses OFF
    '$define _CLKOUT_Function_

'/------------------------------------------------------------------------------
'/ Configuration of the displays hardware.
    '/ Declare Number of Digits (2,3,4,5,6,7,8)
    $define NumberDigits 8

    '/ Declare DecimalP position (0,1,2,3,4,5,6) (0 = No DP)
    $define DecimalP 0

    '/ Declare Shifting Digits option. (YES/NO)
    $define ShiftingDigits YES

'===============================================================================
'/ Declare the Timer used for scanning. (one only)
'    $define ScanTimer0
    $define ScanTimer2

'/ Declare the interruption flags (automatic).
    $ifdef ScanTimer0
    $define DisPlayInterruptFlag INTCONBits_TMR0IF
    $define DisPlayInterruptEnable INTCONBits_TMR0IE
    $endif

    $ifdef ScanTimer2    
    $define DisPlayInterruptFlag PIR1Bits_TMR2IF
    $define DisPlayInterruptEnable PIE1Bits_TMR2IE
    $endif

'/ Declare the HEX number option. (YES/NO)
    $define _HEXnumbers NO    

'/ Declare Hardware type (8 displays with decoder) (= 1 with decoder)
    $define _Hardware 1 
'===============================================================================
'/ Checking security
    $if NumberDigits < 2 Or NumberDigits > 8
    $error "The number of digits must be 2 to 8"
    $endif

'/ Check a timer
    $if !_defined(ScanTimer0) And !_defined(ScanTimer2)
    $error "Timer0 or Timer2 must be declared"    
    $endif    
'===============================================================================
    GoTo _MultiplexDigitsExit
'===============================================================================

'/**** Added by Fuse Configurator ****
'/ Use the Fuse Configurator plug-in to change these settings

Config_Start
$if PLL_ConfigFuses = On
    $if (_xtal >= 16) And (_xtal <= 40)
    OSC = HSPLL	;HS oscillator, PLL enabled (Clock Frequency = 4 x FOSC)(Xtal piece 2,5-10 MHz)
    $else
        $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = On"
    $endif
$endif
$if PLL_ConfigFuses = OFF
    $if (_xtal >= 4) And (_xtal <= 25)
    OSC = HS	    ;HS oscillator PLL disabled  (medium power 4-20 MHz) (Xtal piece 10-32 MHz)
    $else
        $if (_xtal > 25) And (_xtal <= 40)
            $ifdef _CLKOUT_Function_
            OSC = EC     ; Exterior EC oscillator, CLKOUT function On RA6 
            $else
            OSC = ECIO   ; Exterior EC oscillator, port function on RA6 
            $endif
        $else
            $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = OFF"
        $endif
   $endif
$endif
    OSCS = OFF	;Oscillator system clock switch option is disabled (main oscillator is source)
    PWRT = On	;PWRT enabled
    BOR = On	;Brown-out Reset enabled
    BORV = 27	;VBOR set to 2.7V
    WDT = OFF	;WDT disabled (control is placed on the SWDTEN bit)
    WDTPS = 128	;1:128
    CCP2MUX = On	;CCP2 input/output is multiplexed with RC1
    STVR = OFF	;Stack Full/Underflow will not cause RESET
    LVP = OFF	;Low Voltage ICSP disabled
    Debug = OFF	;Background Debugger disabled. RB6 and RB7 configured as general purpose I/O pins.
    Cp0 = OFF	;Block 0 (000200-001FFFh) not code protected
    CP1 = OFF	;Block 1 (002000-003FFFh) not code protected
    CP2 = OFF	;Block 2 (004000-005FFFh) not code protected
    CP3 = OFF	;Block 3 (006000-007FFFh) not code protected
    CPB = OFF	;Boot Block (000000-0001FFh) not code protected
    CPD = OFF	;Data EEPROM not code protected
    WRT0 = OFF	;Block 0 (000200-001FFFh) not write protected
    WRT1 = OFF	;Block 1 (002000-003FFFh) not write protected
    WRT2 = OFF	;Block 2 (004000-005FFFh) not write protected
    WRT3 = OFF	;Block 3 (006000-007FFFh) not write protected
    WRTC = OFF	;Configuration registers (300000-3000FFh) not write protected
    WRTB = OFF	;Boot Block (000000-0001FFh) not write protected
    WRTD = OFF	;Data EEPROM not write protected
    EBTR0 = OFF	;Block 0 (000200-001FFFh) not protected from Table Reads executed in other blocks
    EBTR1 = OFF	;Block 1 (002000-003FFFh) not protected from Table Reads executed in other blocks
    EBTR2 = OFF	;Block 2 (004000-005FFFh) not protected from Table Reads executed in other blocks
    EBTR3 = OFF	;Block 3 (006000-007FFFh) not protected from Table Reads executed in other blocks
    EBTRB = OFF	;Boot Block (000000-0001FFh) not protected from Table Reads executed in other blocks
Config_End
'/**** End of Fuse Configurator Settings ****
'===============================================================================
    On_Hardware_Interrupt GoTo MULTILEX_DIGITS		' Point to the interrupt handler and ENABLE global interrupts
'===============================================================================       

    '/ Using system variables the routines could run faster.
$if NumberDigits = 8
    Dim ONES As Byte System
    Dim TENS As Byte System
    Dim HUNDREDS As Byte System
    Dim THOUSANDS As Byte System
    Dim TENTHOUSANDS As Byte System
    Dim HUNDREDTHOUSANDS As Byte System
    Dim THOUSANDTHOUSANDS As Byte System
    Dim TENTHDTHOUSANDS As Byte System

    $if ShiftingDigits = YES
    Symbol LenghtDISP_ARRAY = 8
    Dim DISP_ARRAY As String * LenghtDISP_ARRAY 
    Dim PRINT_ONES As DISP_ARRAY#0              '/ Variable for interrupt routine.
    Dim PRINT_TENS As DISP_ARRAY#1              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As DISP_ARRAY#2          '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As DISP_ARRAY#3         '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As DISP_ARRAY#4      '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDTHOUSANDS As DISP_ARRAY#5  '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDTHOUSANDS As DISP_ARRAY#6 '/ Variable for interrupt routine.
    Dim PRINT_TENTHDTHOUSANDS As DISP_ARRAY#7   '/ Variable for interrupt routine.
    Dim DISP_ARRAYcop As String * LenghtDISP_ARRAY 
    $endif
    
    $if ShiftingDigits = NO
    Dim PRINT_ONES As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System         '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As Byte System      '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDTHOUSANDS As Byte System  '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDTHOUSANDS As Byte System '/ Variable for interrupt routine.
    Dim PRINT_TENTHDTHOUSANDS As Byte System   '/ Variable for interrupt routine.
    $endif    

    Dim DISPVAL	As Dword System                '/ The number to display on the LEDS
$endif

$if NumberDigits = 7
    Dim ONES As Byte System
    Dim TENS As Byte System
    Dim HUNDREDS As Byte System
    Dim THOUSANDS As Byte System
    Dim TENTHOUSANDS As Byte System
    Dim HUNDREDTHOUSANDS As Byte System
    Dim THOUSANDTHOUSANDS As Byte System

    $if ShiftingDigits = YES
    Symbol LenghtDISP_ARRAY = 7
    Dim DISP_ARRAY As String * LenghtDISP_ARRAY 
    Dim PRINT_ONES As DISP_ARRAY#0              '/ Variable for interrupt routine.
    Dim PRINT_TENS As DISP_ARRAY#1              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As DISP_ARRAY#2          '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As DISP_ARRAY#3         '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As DISP_ARRAY#4      '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDTHOUSANDS As DISP_ARRAY#5  '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDTHOUSANDS As DISP_ARRAY#6 '/ Variable for interrupt routine.
    Dim DISP_ARRAYcop As String * LenghtDISP_ARRAY 
    $endif
    
    $if ShiftingDigits = NO
    Dim PRINT_ONES As Byte System               '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System               '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System           '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As Byte System       '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDTHOUSANDS As Byte System   '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDTHOUSANDS As Byte System  '/ Variable for interrupt routine.
    $endif    
    
    Dim DISPVAL	As Dword System                 '/ The number to display on the LEDS
$endif

$if NumberDigits = 6
    Dim ONES As Byte System
    Dim TENS As Byte System
    Dim HUNDREDS As Byte System
    Dim THOUSANDS As Byte System
    Dim TENTHOUSANDS As Byte System
    Dim HUNDREDTHOUSANDS As Byte System

    $if ShiftingDigits = YES
    Symbol LenghtDISP_ARRAY = 6
    Dim DISP_ARRAY As String * LenghtDISP_ARRAY 
    Dim PRINT_ONES As DISP_ARRAY#0              '/ Variable for interrupt routine.
    Dim PRINT_TENS As DISP_ARRAY#1              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As DISP_ARRAY#2          '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As DISP_ARRAY#3         '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As DISP_ARRAY#4      '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDTHOUSANDS As DISP_ARRAY#5  '/ Variable for interrupt routine.
    Dim DISP_ARRAYcop As String * LenghtDISP_ARRAY
    $endif

    $if ShiftingDigits = NO
    Dim PRINT_ONES As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System         '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As Byte System      '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDTHOUSANDS As Byte System  '/ Variable for interrupt routine.
    $endif

    Dim DISPVAL	As Dword System                '/ The number to display on the LEDS
$endif

$if NumberDigits = 5
    Dim ONES As Byte System
    Dim TENS As Byte System
    Dim HUNDREDS As Byte System
    Dim THOUSANDS As Byte System
    Dim TENTHOUSANDS As Byte System

    $if ShiftingDigits = YES
    Symbol LenghtDISP_ARRAY = 5
    Dim DISP_ARRAY As String * LenghtDISP_ARRAY 
    Dim PRINT_ONES As DISP_ARRAY#0             '/ Variable for interrupt routine.
    Dim PRINT_TENS As DISP_ARRAY#1             '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As DISP_ARRAY#2         '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As DISP_ARRAY#3        '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As DISP_ARRAY#4     '/ Variable for interrupt routine.
    Dim DISP_ARRAYcop As String * LenghtDISP_ARRAY
    $endif
    
    $if ShiftingDigits = NO
    Dim PRINT_ONES As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System         '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As Byte System      '/ Variable for interrupt routine.
    $endif

    Dim DISPVAL	As Dword System                '/ The number to display on the LEDS
$endif

$if NumberDigits = 4
    Dim ONES As Byte System
    Dim TENS As Byte System
    Dim HUNDREDS As Byte System
    Dim THOUSANDS As Byte System

    $if ShiftingDigits = YES
    Symbol LenghtDISP_ARRAY = 4
    Dim DISP_ARRAY As String * LenghtDISP_ARRAY 
    Dim PRINT_ONES As DISP_ARRAY#0             '/ Variable for interrupt routine.
    Dim PRINT_TENS As DISP_ARRAY#1             '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As DISP_ARRAY#2         '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As DISP_ARRAY#3        '/ Variable for interrupt routine.
    Dim DISP_ARRAYcop As String * LenghtDISP_ARRAY
    $endif

    $if ShiftingDigits = NO
    Dim PRINT_ONES As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System         '/ Variable for interrupt routine.
    $endif

    Dim DISPVAL	As Word System                 '/ The number to display on the LEDS
$endif
    
$if NumberDigits = 3
    Dim ONES As Byte System
    Dim TENS As Byte System
    Dim HUNDREDS As Byte System

    $if ShiftingDigits = YES
    Symbol LenghtDISP_ARRAY = 3
    Dim DISP_ARRAY As String * LenghtDISP_ARRAY 
    Dim PRINT_ONES As DISP_ARRAY#0           '/ Variable for interrupt routine.
    Dim PRINT_TENS As DISP_ARRAY#1
    Dim PRINT_HUNDREDS As DISP_ARRAY#2
    Dim DISP_ARRAYcop As String * LenghtDISP_ARRAY
    $endif

    $if ShiftingDigits = NO
    Dim PRINT_ONES As Byte System           '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System
    Dim PRINT_HUNDREDS As Byte System
    $endif

    Dim DISPVAL	As Word System             '/ The number to display on the LEDS
$endif

$if NumberDigits = 2
    '/ NO shift for 2 digits.
    Dim ONES As Byte System
    Dim TENS As Byte System
    Dim PRINT_ONES As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System

    Dim DISPVAL	As Byte System             '/ The number to display on the LEDS
$endif
        
    $if NumberDigits = 8
    Dim TIMESHARE As Word System    '/ Variable Used by the interrupt for time sharing
    $else
    Dim TIMESHARE As Byte System	'/ Variable Used by the interrupt for time sharing    
    $endif

$ifndef FSR0_Var
    $define FSR0_Var
	Dim FSR0 As FSR0L.Word         'Combine FSR0L\H into a single 16-bit register
$endif
$ifndef FSR1_Var
    $define FSR1_Var
	Dim FSR1 As FSR1L.Word         'Combine FSR1L\H into a single 16-bit register
$endif

'/ Temporal variables

'/------------------------------------------------------------------------------
    '/ Definition of the PORTS for Display PORT_DIGITS.
    Symbol PORT_DIGITS = LATD

    '/ Definition of the pins to enable displays to write.
    Symbol DIGIT8 = LATD.5
    Symbol DISP_DP = LATD.4 
    Symbol LE2 = LATD.6 
    Symbol LE1 = LATD.7
        
'===============================================================================
'===============================================================================
'/ INTERRUPT HANDLER 
'/ Multiplexing the 8-digits
'/
MULTILEX_DIGITS:
Context Save				
    If DisPlayInterruptFlag = 1 Then 		

        '/ Process the first display [1]
        If TIMESHARE.0 = 1 Then         '/ If it's our turn then...
            PRINT_ONES.7 = 1            '/ Rewrite the LE1 Bit, segments decoder Disable.
            PORT_DIGITS = PRINT_ONES    '/ Write the digit value, clear LE2 & the DIGIT8 enable.  
            LE1 = 0                     '/ Segments value is stable.
            DelayUS 1                   '/ The delay could be less value, see the datasheet of the 74HC4511.
            LE1 = 1                     '/ Write to the segments decoder.
            DelayCS 3                   '/ Waiting PORT stable. Adjust the delay value with the 74HC238 parameters.
            PORT_DIGITS = %11000001     '/ Number of digit to scan & Set LE2.

        '/ Process the second display [2]									
        ElseIf TIMESHARE.1 = 1 Then     '/ If it's our turn then...
            PRINT_TENS.7 = 1            '/ Set up the pattern for PORTD  
            PORT_DIGITS = PRINT_TENS    '/  Write the digit value, Clear LE2 & the DIGIT8 Enable.
            LE1 = 0                     
            DelayUS 1
            LE1 = 1                     '/ Write To the segments decoder.
            DelayCS 3                   '/ Waiting PORT stable. Adjust the delay value with the 74HC238 parameters.
            $if DecimalP = 1 
            PORT_DIGITS = %11010010     '/ Number of digit To scan, Set DP & Set LE2.
            $else
            PORT_DIGITS = %11000010     '/ Number of digit To scan & Set LE2.
            $endif

        '/ Process the third display [3]									
        $if NumberDigits > 2
        ElseIf TIMESHARE.2 = 1 Then         '/ If it's our turn then...
            PRINT_HUNDREDS.7 = 1
            PORT_DIGITS = PRINT_HUNDREDS    '/  Write the digit value, Clear LE2 & the DIGIT8 Enable.
            LE1 = 0
            DelayUS 1
            LE1 = 1                         '/ Write To the segments decoder.
            DelayCS 3                       '/ Waiting PORT stable. Adjust the delay value with the 74HC238 parameters.
            $if DecimalP = 2 
            PORT_DIGITS = %11010011         '/ Number of digit To scan, set DP & Set LE2.
            $else
            PORT_DIGITS = %11000011         '/ Number of digit To scan & Set LE2.
            $endif
        $endif
                
        '/ Process the fourth display [4]			
        $if NumberDigits > 3
        ElseIf TIMESHARE.3 = 1 Then         '/ If it's our turn then...
            PRINT_THOUSANDS.7 = 1
            PORT_DIGITS = PRINT_THOUSANDS   '/  Write the digit value, Clear LE2 & the DIGIT8 Enable.
            LE1 = 0
            DelayUS 1
            LE1 = 1                         '/ Write To the segments decoder.
            DelayCS 3                       '/ Waiting PORT stable. Adjust the delay value with the 74HC238 parameters.
            $if DecimalP = 3 
            PORT_DIGITS = %11010100         '/ Number of digit To scan, Set DP & Set LE2.
            $else
            PORT_DIGITS = %11000100         '/ Number of digit To scan & Set LE2.
            $endif
        $endif

        '/ Process the fifht display [5]			
        $if NumberDigits > 4
        ElseIf TIMESHARE.4 = 1 Then             '/ If it's our turn then...
            PRINT_TENTHOUSANDS.7 = 1
            PORT_DIGITS = PRINT_TENTHOUSANDS    '/  Write the digit value, Clear LE2 & the DIGIT8 Enable.
            LE1 = 0
            DelayUS 1
            LE1 = 1                             '/ Write To the segments decoder.
            DelayCS 3                           '/ Waiting PORT stable. Adjust the delay value with the 74HC238 parameters.
            $if DecimalP = 4 
            PORT_DIGITS = %11010101             '/ Number of digit To scan, Set DP & Set LE2.
            $else
            PORT_DIGITS = %11000101             '/ Number of digit To scan & Set LE2.
            $endif
        $endif

        '/ Process the sixfht display [6]			
        $if NumberDigits > 5
        ElseIf TIMESHARE.5 = 1 Then                 '/ If it's our turn then...
            PRINT_HUNDREDTHOUSANDS.7 = 1            '/ Rewrite the LE1 bit.
            PORT_DIGITS = PRINT_HUNDREDTHOUSANDS    '/  Write the digit value, Clear LE2 & the DIGIT8 Enable.
            LE1 = 0
            DelayUS 1
            LE1 = 1                     '/ Write To the segments decoder.
            DelayCS 3                   '/ Waiting PORT stable. Adjust the delay value with the 74HC238 parameters.
            $if DecimalP = 5 
            PORT_DIGITS = %11010110     '/ Number of digit To scan, Set DP & Set LE2.
            $else
            PORT_DIGITS = %11000110     '/ Number of digit To scan & Set LE2.
            $endif
        $endif

        '/ Process the sevenfht display [7]			
        $if NumberDigits > 6
        ElseIf TIMESHARE.6 = 1 Then                 '/ If it's our turn then...
            PRINT_THOUSANDTHOUSANDS.7 = 1           '/ Rewrite the LE1 Bit, segments decoder Disable.
            PORT_DIGITS = PRINT_THOUSANDTHOUSANDS   '/  Write the digit value, Clear LE2 & the DIGIT8 Enable.
            LE1 = 0                                 '/ Segments value is stable.
            DelayUS 1                               '/ The delay could be less value, see the datasheet of the 74HC4511.
            LE1 = 1                                 '/ Write To the segments decoder.
            DelayCS 3                               '/ Waiting PORT stable. Adjust the delay value with the 74HC238 parameters.
            $if DecimalP = 6 
            PORT_DIGITS = %11010111     '/ Number of digit To scan, Set DP & Set LE2.
            $else
            PORT_DIGITS = %11000111     '/ Number of digit To scan & Set LE2.
            $endif
        $endif

        '/ Process the eightfht display [8]	(special format)		
        $if NumberDigits > 7
        ElseIf TIMESHARE.7 = 1 Then                 '/ If it's our turn then...
            PRINT_TENTHDTHOUSANDS.7 = 1             '/ Rewrite the LE1 bit, segments decoder disable.
            PORT_DIGITS = PRINT_TENTHDTHOUSANDS     '/ %1000xxxx Write the digit value, Clear LE2 & the DIGIT8 Enable.
            LE1 = 0                                 '/ Segments value is stable.
            DelayUS 1                               '/ The delay could be less value, see the datasheet of the 74HC4511.
            LE1 = 1                                 '/ Write To the segments decoder.
            DelayCS 3                               '/ Waiting PORT stable. Adjust the delay value with the 74HC238 parameters.
            '/ Clear the DA/DB/DC/DD pins because the 74HC238 is not used. Mantain LE1. Clear LE2 & set DIGIT8.
            $if DecimalP = 7 
            PORT_DIGITS = %10110000     '/ Mantain LE1, Set DIGIT8, set DP & Clear LE2 (not use the 74HC238).
            $else
            PORT_DIGITS = %10100000     '/ Mantain LE1, Set DIGIT8, clear DP & clear LE2 (not use the 74HC238).
            $endif
        $endif

        EndIf
    
        '/ Process the scanning.
        TIMESHARE = TIMESHARE << 1      '/ Shift bit to set the next digit display.

        '/ Checking the last digit & reset scanning.        
        $if NumberDigits = 8
        If TIMESHARE.8 = 1 Then			'/ Is bit 8 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 7
        If TIMESHARE.7 = 1 Then			'/ Is bit 7 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 6
        If TIMESHARE.6 = 1 Then			'/ Is bit 6 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 5
        If TIMESHARE.5 = 1 Then			'/ Is bit 5 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 4
        If TIMESHARE.4 = 1 Then			'/ Is bit 4 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 3
        If TIMESHARE.3 = 1 Then			'/ Is bit 3 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif        
        $if NumberDigits = 2
        If TIMESHARE.2 = 1 Then			'/ Is bit 2 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        DisPlayInterruptFlag = 0        '/ Reset TMR0 interrupt flag
    EndIf

Context Restore					        '/ Exit the interrupt routine		

'===============================================================================
'===============================================================================		
'/ Build up the seperate digits of variable DISPVAL
$define PrintDigits() GoSub BuildDigits_End


$define BuildDigits(pValue) '
    DISPVAL = pValue        '
    _mBuildDigits pValue

'/ This macro is used for testing variables only.
_mBuildDigits Macro- pValue
    #if (Prm_Count != 1)
        #error "_mPrintDispInt - This macro uses 1 parameter!"
    #else
        #if (Prm_1 != Dword) && (Prm_1 != Word) && (Prm_1 != Byte)
            #error "_mBuildDigits Macro - (Param1) Should be a Dword, Word or Byte Variable"
        #endif
    #endif
    GoSub Build_Digits_Sub
Endm

Build_Digits_Sub:
    #ifdef watchdog_req
    Clrwdt
    #endif
    '/--------------------------------------------------------------------------
    '/ Built Digit [1] ONES
    ONES = Dig DISPVAL,0
    '/--------------------------------------------------------------------------
    '/ Built Digit [2] TENS
    If DISPVAL < 10 Then
        $if NumberDigits = 8 
    	TENS = 15                 '/ Blank the TENS DIGIT
    	HUNDREDS = 15			  '/ Blank the HUNDREDS DIGIT
    	THOUSANDS = 15			  '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 15	      '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 15     '/ Blank the TENTHOUSANDS DIGIT
    	THOUSANDTHOUSANDS = 15    '/ Blank the THOUSANDTHOUSANDS DIGIT
        TENTHDTHOUSANDS = 15      '/ Blank the TENTHDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 7 
    	TENS = 15                 '/ Blank the TENS DIGIT
    	HUNDREDS = 15			  '/ Blank the HUNDREDS DIGIT
    	THOUSANDS = 15			  '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 15	      '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 15     '/ Blank the TENTHOUSANDS DIGIT
        THOUSANDTHOUSANDS = 15    '/ Blank the THOUSANDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 6 
    	TENS = 15                 '/ Blank the TENS DIGIT
    	HUNDREDS = 15			  '/ Blank the HUNDREDS DIGIT
    	THOUSANDS = 15			  '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 15	      '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 15     '/ Blank the TENTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 5 
    	TENS = 15                '/ Blank the TENS DIGIT
    	HUNDREDS = 15			 '/ Blank the HUNDREDS DIGIT
    	THOUSANDS = 15			 '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 15	     '/ Blank the TENTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 4 
    	TENS = 15                '/ Blank the TENS DIGIT
    	HUNDREDS = 15			 '/ Blank the HUNDREDS DIGIT
    	THOUSANDS = 15			 '/ Blank the THOUSANDS DIGIT
        $endif
        $if NumberDigits = 3 
    	TENS = 15                '/ Blank the TENS DIGIT
    	HUNDREDS = 15			 '/ Blank the HUNDREDS DIGIT
        $endif
        $if NumberDigits = 2 
    	TENS = 15                '/ Blank the TENS DIGIT
        $endif
        $if NumberDigits > 2
        GoTo BuildDigits_End     '/ No more digits to check.
        $endif
    Else
        TENS = Dig DISPVAL,1
    EndIf
    '/--------------------------------------------------------------------------    
    '/ Built Digit [3] HUNDREDS
    $if NumberDigits > 2 
    If DISPVAL < 100 Then
        $if NumberDigits = 8 
        HUNDREDS = 15             '/ Blank the HUNDREDS DIGIT
    	THOUSANDS = 15			  '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 15	      '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 15     '/ Blank the HUNDREDTHOUSANDS DIGIT
    	THOUSANDTHOUSANDS = 15    '/ Blank the THOUSANDTHOUSANDS DIGIT
        TENTHDTHOUSANDS = 15      '/ Blank the TENTHDTHOUSANDS DIGIT
        $endif
        $if NumberDigits = 7 
    	HUNDREDS = 15			  '/ Blank the HUNDREDS DIGIT
    	THOUSANDS = 15			  '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 15	      '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 15     '/ Blank the TENTHOUSANDS DIGIT
        THOUSANDTHOUSANDS = 15    '/ Blank the THOUSANDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 6 
    	HUNDREDS = 15			  '/ Blank the HUNDREDS DIGIT
    	THOUSANDS = 15			  '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 15	      '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 15     '/ Blank the TENTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 5
    	HUNDREDS = 15			 '/ Blank the HUNDREDS DIGIT
    	THOUSANDS = 15			 '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 15	     '/ Blank the TENTHOUSANDS DIGIT
        $endif
        $if NumberDigits = 4
    	HUNDREDS = 15			 '/ Blank the HUNDREDS DIGIT
    	THOUSANDS = 15			 '/ Blank the THOUSANDS DIGIT
        $endif
        $if NumberDigits = 3
    	HUNDREDS = 15			 '/ Blank the HUNDREDS DIGIT
        $endif        
        $if NumberDigits > 3
        GoTo BuildDigits_End     '/ No more digits to check.
        $endif
    Else        	
        HUNDREDS = Dig DISPVAL,2
    EndIf
    $endif
    '/--------------------------------------------------------------------------
    '/ Built Digit [4] THOUSANDS
    $if NumberDigits > 3    
    If DISPVAL < 1000 Then
        $if NumberDigits = 8 
        THOUSANDS = 15            '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 15	      '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 15     '/ Blank the HUNDREDTHOUSANDS DIGIT
    	THOUSANDTHOUSANDS = 15    '/ Blank the THOUSANDTHOUSANDS DIGIT
        TENTHDTHOUSANDS = 15      '/ Blank the TENTHDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 7 
    	THOUSANDS = 15			  '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 15	      '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 15     '/ Blank the HUNDREDTHOUSANDS DIGIT
        THOUSANDTHOUSANDS = 15    '/ Blank the THOUSANDTHOUSANDS DIGIT
        $endif
        $if NumberDigits = 6 
    	THOUSANDS = 15			  '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 15	      '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 15     '/ Blank the HUNDREDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 5
    	THOUSANDS = 15			  '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 15	      '/ Blank the TENTHOUSANDS DIGIT
        $endif
        $if NumberDigits = 4
    	THOUSANDS = 15			 '/ Blank the THOUSANDS DIGIT
        $endif
        $if NumberDigits > 4
        GoTo BuildDigits_End     '/ No more digits to check.
        $endif
    Else        	
        THOUSANDS = Dig DISPVAL,3 
    EndIf			
    $endif
    '/--------------------------------------------------------------------------
    '/ Built Digit [5] TENTHOUSANDS
    $if NumberDigits > 4   
    If DISPVAL < 10000 Then
        $if NumberDigits = 8 
    	TENTHOUSANDS = 15	     '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 15    '/ Blank the HUNDREDTHOUSANDS DIGIT
        THOUSANDTHOUSANDS = 15   '/ Blank the THOUSANDTHOUSANDS DIGIT
        TENTHDTHOUSANDS = 15     '/ Blank the TENTHDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 7 
    	TENTHOUSANDS = 15	     '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 15    '/ Blank the HUNDREDTHOUSANDS DIGIT
        THOUSANDTHOUSANDS = 15   '/ Blank the THOUSANDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 6 
    	TENTHOUSANDS = 15	     '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 15    '/ Blank the HUNDREDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 5
    	TENTHOUSANDS = 15        '/ Blank the TENTHOUSANDS DIGIT
        $endif
        $if NumberDigits > 5
        GoTo BuildDigits_End     '/ No more digits to check.
        $endif
    Else        	
        TENTHOUSANDS = Dig DISPVAL,4 
    EndIf			
    $endif
    '/--------------------------------------------------------------------------
    '/ Built Digit [6] HUNDREDTHOUSANDS
    $if NumberDigits > 5   
    If DISPVAL < 100000 Then
        $if NumberDigits = 8 
        HUNDREDTHOUSANDS = 15       '/ Blank the HUNDREDTHOUSANDS DIGIT
    	THOUSANDTHOUSANDS = 15      '/ Blank the THOUSANDTHOUSANDS DIGIT
        TENTHDTHOUSANDS = 15        '/ Blank the TENTHDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 7 
        HUNDREDTHOUSANDS = 15       '/ Blank the HUNDREDTHOUSANDS DIGIT
    	THOUSANDTHOUSANDS = 15      '/ Blank the THOUSANDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 6
        HUNDREDTHOUSANDS = 15       '/ Blank the HUNDREDTHOUSANDS DIGIT
        $endif       
        $if NumberDigits > 6
        GoTo BuildDigits_End        '/ No more digits to check.
        $endif
    Else        	
        HUNDREDTHOUSANDS = Dig DISPVAL,5
    EndIf			
    $endif
    '/--------------------------------------------------------------------------
    '/ Built Digit [7] THOUSANDTHOUSANDS
    $if NumberDigits > 6   
    If DISPVAL < 1000000 Then
        $if NumberDigits = 8 
        THOUSANDTHOUSANDS = 15      '/ Blank the THOUSANDTHOUSANDS DIGIT
        TENTHDTHOUSANDS = 15        '/ Blank the TENTHDTHOUSANDS DIGIT
        $endif
        $if NumberDigits = 7 
        THOUSANDTHOUSANDS = 15      '/ Blank the THOUSANDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits > 7
        GoTo BuildDigits_End        '/ No more digits to check.
        $endif
    Else        	
        THOUSANDTHOUSANDS = Dig DISPVAL,6
    EndIf			
    $endif
    '/--------------------------------------------------------------------------
    '/ Built Digit [8] TENTHDTHOUSANDS
    $if NumberDigits > 7
    If DISPVAL < 10000000 Then
        TENTHDTHOUSANDS = 15            '/ Blank the TENTHDTHOUSANDS DIGIT
    Else
        TENTHDTHOUSANDS = Dig DISPVAL,7
    EndIf
    $endif
    '/--------------------------------------------------------------------------
BuildDigits_End:
    #ifdef watchdog_req
    Clrwdt
    #endif
    '/ Prepare Display 8 digits.
    $if NumberDigits = 8
        PRINT_ONES = ONES
        PRINT_TENS = TENS
        PRINT_HUNDREDS = HUNDREDS
        PRINT_THOUSANDS = THOUSANDS        
        PRINT_TENTHOUSANDS = TENTHOUSANDS 
        PRINT_HUNDREDTHOUSANDS = HUNDREDTHOUSANDS 
        PRINT_THOUSANDTHOUSANDS = THOUSANDTHOUSANDS
        PRINT_TENTHDTHOUSANDS = TENTHDTHOUSANDS
    $endif

    '/ Prepare Display 7 digits.
    $if NumberDigits = 7
        ' Make a copy to print the values for the interrupt routine.
        DisPlayInterruptEnable = 0           '/ Avoid blinking.
        PRINT_ONES = ONES
        PRINT_TENS = TENS
        PRINT_HUNDREDS = HUNDREDS
        PRINT_THOUSANDS = THOUSANDS        
        PRINT_TENTHOUSANDS = TENTHOUSANDS 
        PRINT_HUNDREDTHOUSANDS = HUNDREDTHOUSANDS 
        PRINT_THOUSANDTHOUSANDS = THOUSANDTHOUSANDS
        DisPlayInterruptEnable = 1
    $endif

    '/ Prepare Display 6 digits.
    $if NumberDigits = 6
        ' Make a copy to print the values for the interrupt routine.
        DisPlayInterruptEnable = 0           '/ Avoid blinking.
        PRINT_ONES = ONES
        PRINT_TENS = TENS
        PRINT_HUNDREDS = HUNDREDS
        PRINT_THOUSANDS = THOUSANDS        
        PRINT_TENTHOUSANDS = TENTHOUSANDS 
        PRINT_HUNDREDTHOUSANDS = HUNDREDTHOUSANDS 
        DisPlayInterruptEnable = 1
    $endif

    '/ Prepare Display 5 digits.
    $if NumberDigits = 5
        ' Make a copy to print the values for the interrupt routine.
        DisPlayInterruptEnable = 0           '/ Avoid blinking.
        PRINT_ONES = ONES
        PRINT_TENS = TENS
        PRINT_HUNDREDS = HUNDREDS
        PRINT_THOUSANDS = THOUSANDS        
        PRINT_TENTHOUSANDS = TENTHOUSANDS 
        DisPlayInterruptEnable = 1
    $endif

    '/ Prepare Display 4 digits.
    $if NumberDigits = 4
        ' Make a copy to print the values for the interrupt routine.
        DisPlayInterruptEnable = 0           '/ Avoid blinking.
        PRINT_ONES = ONES
        PRINT_TENS = TENS
        PRINT_HUNDREDS = HUNDREDS
        PRINT_THOUSANDS = THOUSANDS        
        DisPlayInterruptEnable = 1
    $endif

    '/ Prepare Display 3 digits.
    $if NumberDigits = 3
        ' Make a copy to print the values for the interrupt routine.
        DisPlayInterruptEnable = 0           '/ Avoid blinking.
        PRINT_ONES = ONES
        PRINT_TENS = TENS
        PRINT_HUNDREDS = HUNDREDS
        DisPlayInterruptEnable = 1
    $endif

    '/ Prepare Display 2 digits.
    $if NumberDigits = 2
        ' Make a copy to print the values for the interrupt routine.
        DisPlayInterruptEnable = 0           '/ Avoid blinking.
        PRINT_ONES = ONES
        PRINT_TENS = TENS
        DisPlayInterruptEnable = 1
    $endif
      
    Return    
'===============================================================================
'/ Subroutine: Init the displays
'/
$ifdef ScanTimer0
$define DisplayInit() GoSub DisplayInit_Sub

DisplayInit_Sub:
'/ CONFIGURATION OF TIMER0
'/ Set TMR0 to interrupt on rollover
'/ The TMR0H & TMR0l counters are not loaded to simplify the code. Then the timing
'/ could change in every Delare Xtal.
'/
    #ifdef watchdog_req
    Clrwdt
    #endif
    WDTCON = 0          '/ Without watchdog

    INTCON = 0                                              '/ Clear all interrupts
    While INTCONBits_GIE = 1 : INTCONBits_GIE = 0 : Wend	'/ Make sure global interrupts are off
    INTCONBits_TMR0IE = 0       '/ Disable the Timer0 interrupts.

    INTCON2Bits_RBPU = 1        '/ All PORTB pull-ups are disabled.
    RCONBits_IPEN = 1           '/ Enable priority levels on interrupts.
    INTCON2Bits_TMR0IP = 1      '/ Timer0 for High Interrupt.

    '/ Configure the Timer0 for FOSC = 4Mhz.
    $if _xtal = 4
    T0CON = %00000100           '/ div32 => 8,1 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    '/ Configure the Timer0 for FOSC = 8Mhz used for Proteus.
    $if NumberDigits = 8 Or NumberDigits = 7
        $if _xtal = 8
        T0CON = %00000100           '/ div32 => 4,0 mS Timer0 Prescaler Select bits 
        T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
        T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
        T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
        $endif
    $else
        $if _xtal = 8
        T0CON = %00000101           '/ div64 => 8,1 mS Timer0 Prescaler Select bits 
        T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
        T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
        T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
        $endif   
    $endif

    '/ Configure the Timer0 for FOSC = 16Mhz.
    $if _xtal = 16
    T0CON = %00000110           '/ div128 => 8,1 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    '/ Configure the Timer0 for FOSC = 20Mhz.
    $if _xtal = 20
    T0CON = %00000110           '/ div128 => 6,55 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    '/ Configure the Timer0 for FOSC = 32Mhz.
    $if _xtal = 32
    T0CON = %00000111           '/ div256 => 8,1 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    '/ Configure the Timer0 for FOSC = 40Mhz.
    $if _xtal = 40
    T0CON = %00000000           '/ div2 => 13,1 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 0        '/ 0 = Timer0 is configured as an 16-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    '/ Configure the Timer0 for FOSC = 64Mhz for Kxx PICs.
    $if _xtal = 64
    T0CON = %00000001           '/ div4 => 8,2 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 0        '/ 0 = Timer0 is configured as an 16-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    '/ Configure the Timer0 for FOSC = 80Mhz for K20 Pics.
    $if _xtal = 80
    T0CON = %00000000           '/ div2 => 6,55 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 0        '/ 0 = Timer0 is configured as an 16-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    TMR0H = 0                   '/ Clear the Timer0 counter.
    Nop
    TMR0L = 0
    Nop    
    T0CONBits_TMR0ON = 1        '/ Start the Timer0.
    '---------------------------------------------------------------------------
    '/ Set the PORTS & Pins to digital.
    ADCON1 = 7
	Output PORT_DIGITS          '/ Put the display PORT as Output.

    ' Write the lines OFF at reset depending the Display Mode.
    ' Blanks directly the Port at reset depending of the option.
    PORT_DIGITS = 0             '/ All displays cleared for common cathode.
	TIMESHARE = 1		        '/ Reset the time share variable

    '/ Blanks the Displays at reset.
    $if NumberDigits = 8
        PRINT_ONES = 15                '/ Blank the ONES DIGIT
        PRINT_TENS = 15                '/ Blank the TENS DIGIT
        PRINT_HUNDREDS = 15            '/ Blank the HUNDREDS DIGIT
        PRINT_THOUSANDS = 15           '/ Blank the THOUSANDS DIGIT
        PRINT_TENTHOUSANDS = 15        '/ Blank the TENTHOUSANDS DIGIT
        PRINT_HUNDREDTHOUSANDS = 15    '/ Blank the TENTHOUSANDS DIGIT
        PRINT_THOUSANDTHOUSANDS = 15   '/ Blank the THOUSANDTHOUSANDS DIGIT
        PRINT_TENTHDTHOUSANDS = 15     '/ Blank the TENTHDTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 7
        PRINT_ONES = 15                '/ Blank the ONES DIGIT
        PRINT_TENS = 15                '/ Blank the TENS DIGIT
        PRINT_HUNDREDS = 15            '/ Blank the HUNDREDS DIGIT
        PRINT_THOUSANDS = 15           '/ Blank the THOUSANDS DIGIT
        PRINT_TENTHOUSANDS = 15        '/ Blank the TENTHOUSANDS DIGIT
        PRINT_HUNDREDTHOUSANDS = 15    '/ Blank the TENTHOUSANDS DIGIT
        PRINT_THOUSANDTHOUSANDS = 15   '/ Blank the THOUSANDTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 6
        PRINT_ONES = 15                '/ Blank the ONES DIGIT
        PRINT_TENS = 15                '/ Blank the TENS DIGIT
        PRINT_HUNDREDS = 15            '/ Blank the HUNDREDS DIGIT
        PRINT_THOUSANDS = 15           '/ Blank the THOUSANDS DIGIT
        PRINT_TENTHOUSANDS = 15        '/ Blank the TENTHOUSANDS DIGIT
        PRINT_HUNDREDTHOUSANDS = 15    '/ Blank the HUNDREDTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 5
        PRINT_ONES = 15            '/ Blank the ONES DIGIT
        PRINT_TENS = 15            '/ Blank the TENS DIGIT
        PRINT_HUNDREDS = 15        '/ Blank the HUNDREDS DIGIT
        PRINT_THOUSANDS = 15       '/ Blank the THOUSANDS DIGIT
        PRINT_TENTHOUSANDS = 15    '/ Blank the TENTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 4
        PRINT_ONES = 15        '/ Blank the ONES DIGIT
        PRINT_TENS = 15        '/ Blank the TENS DIGIT
        PRINT_HUNDREDS = 15    '/ Blank the HUNDREDS DIGIT
        PRINT_THOUSANDS = 15   '/ Blank the THOUSANDS DIGIT
    $endif
    $if NumberDigits = 3
        PRINT_ONES = 15        '/ Blank the ONES DIGIT
        PRINT_TENS = 15        '/ Blank the TENS DIGIT
        PRINT_HUNDREDS = 15    '/ Blank the HUNDREDS DIGIT
    $endif
    $if NumberDigits = 2
        PRINT_ONES = 15        '/ Blank the ONES DIGIT
        PRINT_TENS = 15        '/ Blank the TENS DIGIT
    $endif    

    ' Configure the interrupts to start.
    INTCONBits_TMR0IF = 0   '/ Clear the Timer0 interrupt flag to begin.
    INTCONBits_TMR0IE = 1   '/ Enable the Timer0 interrupts.
	INTCONBits_GIE = 1	    '/ Enables all High priority interrupts
    ' The Timer0 is running...
    Return
$endif
'---------------------------------------------------------------------------------------
$ifdef ScanTimer2
'/ Subroutine: Init the displays
$define DisplayInit() GoSub DisplayInit_Sub

DisplayInit_Sub:
'/ CONFIGURATION OF TIMER2
'/ Set TMR2 to interrupt on rollover
'/ The TMR2 counter is not loaded.
'/
    #ifdef watchdog_req
    Clrwdt
    #endif
    WDTCON = 0          '/ Without watchdog
    
'/ Same structure for 18F2xk22 & 18F2xk20 & 18F452 series.
    T2CON = 0
    INTCON = 0                                              '/ Clear all interrupts
    While INTCONBits_GIE = 1 : INTCONBits_GIE = 0 : Wend	'/ Make sure global interrupts are off
    PIE1Bits_TMR2IE = 0         '/ Disable the Timer2 interrupts.

    INTCON2Bits_RBPU = 1        '/ All PORTB pull-ups are disabled.
    RCONBits_IPEN = 1           '/ Enable priority levels on interrupts.
    IPR1Bits_TMR2IP = 1         '/ Timer2 for High Interrupt.

    $if _xtal = 4
        ' TMR2 8Bits counter, Postscaler 1/10, Prescaler 1/4, Clock Xtal/4 (Xtal: Xtal 4 Mhz)
        ' For Timer2 interrupt every 10,0 mS 
        T2CON = %01001001
        PR2 = 248
    $endif

    '/ used For Proteus
    $if NumberDigits = 8 Or NumberDigits = 7
        $if _xtal = 8
            ' TMR2 8Bits counter, Postscaler 1/2, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 8 Mhz)
            ' For Timer2 interrupt every 3,95 mS 
            T2CON = %00001011
            PR2 = 249
        $endif
    $else
        $if _xtal = 8
            ' TMR2 8Bits counter, Postscaler 1/5, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 8 Mhz)
            ' For Timer2 interrupt every 8,0 mS 
            T2CON = %00100011
            PR2 = 249
        $endif
    $endif

    $if _xtal = 16
        ' TMR2 8Bits counter, Postscaler 1/16, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 4 + PLL = 16 Mhz) or (Xtal: Xtal 16Mhz)
        ' For Timer2 interrupt every 10,0 mS 
        T2CON = %01111011
        PR2 = 155        
    $endif

    $if _xtal = 20
        ' TMR2 8Bits counter, Postscaler 1/13, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 20 Mhz)
        ' For Timer2 interrupt every 10,00 mS 
        T2CON = %01100011
        PR2 = 239
    $endif

    $if _xtal = 32
        ' TMR2 8Bits counter, Postscaler 1/4, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 20 Mhz)
        ' For Timer2 interrupt every 8,2 mS 
        T2CON = %01111011
        PR2 = 255
    $endif

    $if _xtal = 40
        ' TMR2 8Bits counter, Postscaler 1/16, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 20 Mhz) or (Xtal: Xtal 10 Mhz + PLL)
        ' For Timer2 interrupt every 6,6 mS 
        T2CON = %01111011
        PR2 = 255
    $endif

    $if _xtal = 64
        ' TMR2 8Bits counter, Postscaler 1/16, Prescaler 1/16, Osc Off, Clock Xtal/4 (Xtal: Xtal 16 + PLL = 64Mhz)
        ' For Timer2 interrupt every 4,2 mS
        T2CON = %01111011
        PR2 = 255
    $endif

    ' Updated for Xtal = 80 Mhz, but for testing only because the Timing is only 3,2 mS.
    ' For K20 series.
    $if _xtal = 80
        ' TMR2 8Bits counter, Postscaler 1/16, Prescaler 1/16, Osc Off, Clock Xtal/4 (Xtal: Xtal 20 + PLL = 80Mhz)
        ' For Timer2 interrupt every 3,3 mS
        T2CON = %01111011
        PR2 = 255
    $endif       
       
    TMR2 = 0                        '/ Clear the Timer2 counter.
    Nop    
    T2CONBits_TMR2ON = 1            '/ Start the Timer2.
    '---------------------------------------------------------------------------
    '/ Set the PORTS & Pins to digital.
    ADCON1 = 7
	Output PORT_DIGITS             '/ Put the display PORT as Output.

    ' Write the lines OFF at reset depending the Display Mode.
    ' Blanks directly the Port at reset depending of the option.
    PORT_DIGITS = 0                '/ PORT cleared.
	TIMESHARE = 1		           '/ Reset the time share variable

    '/ Blanks the Displays at reset.
    $if NumberDigits = 8
    PRINT_ONES = 15                '/ Blank the ONES DIGIT
    PRINT_TENS = 15                '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 15            '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 15           '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 15        '/ Blank the TENTHOUSANDS DIGIT
    PRINT_HUNDREDTHOUSANDS = 15    '/ Blank the TENTHOUSANDS DIGIT
    PRINT_THOUSANDTHOUSANDS = 15   '/ Blank the THOUSANDTHOUSANDS DIGIT
    PRINT_TENTHDTHOUSANDS = 15     '/ Blank the TENTHDTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 7
    PRINT_ONES = 15                '/ Blank the ONES DIGIT
    PRINT_TENS = 15                '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 15            '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 15           '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 15        '/ Blank the TENTHOUSANDS DIGIT
    PRINT_HUNDREDTHOUSANDS = 15    '/ Blank the TENTHOUSANDS DIGIT
    PRINT_THOUSANDTHOUSANDS = 15   '/ Blank the THOUSANDTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 6
    PRINT_ONES = 15                '/ Blank the ONES DIGIT
    PRINT_TENS = 15                '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 15            '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 15           '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 15        '/ Blank the TENTHOUSANDS DIGIT
    PRINT_HUNDREDTHOUSANDS = 15    '/ Blank the TENTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 5
    PRINT_ONES = 15            '/ Blank the ONES DIGIT
    PRINT_TENS = 15            '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 15        '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 15       '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 15    '/ Blank the TENTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 4
    PRINT_ONES = 15        '/ Blank the ONES DIGIT
    PRINT_TENS = 15        '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 15    '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 15   '/ Blank the THOUSANDS DIGIT
    $endif
    $if NumberDigits = 3
    PRINT_ONES = 15        '/ Blank the ONES DIGIT
    PRINT_TENS = 15        '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 15    '/ Blank the HUNDREDS DIGIT
    $endif
    $if NumberDigits = 2
    PRINT_ONES = 15        '/ Blank the ONES DIGIT
    PRINT_TENS = 15        '/ Blank the TENS DIGIT
    $endif    

    ' Configure the interrupts to start.
    PIR1Bits_TMR2IF = 0     '/ Clear the Timer0 interrupt flag to begin.
    PIE1Bits_TMR2IE = 1     '/ Enable the Timer0 interrupts.
	INTCONBits_GIE = 1	    '/ Enables all High priority interrupts
    ' The Timer2 is running...
    Return
$endif
'/==============================================================================
'/ Shifting Macros:
'/ The Original data is in DISP_ARRAY.
'/ The new data shifted will be in the DISP_ARRAYcop.
'/ The FSR0 pointer is used for the DISP_ARRAY.
'/ The FSR1 pointer is used for the DISP_ARRAYcop. 
'/
'/ First parameter: Number of display shifts.
'/ Second parameter: Value of delay in mS between shift.
'/
$if ShiftingDigits = YES

#ifMacro- _mRotateLeftDispArray, _mRotateRightDispArray 
    Dim RotateDelay As Word System
    Dim DispLenArray As Byte System
#endIfMacro-

$define RotateRightDisplay(pNumberShift, pDelayMS) '
    $if NumberDigits < 3                           '
    $error "Too few digits for shifting"           '
    $endif                                         '
    $if pNumberShift = 0                           '
    $error "Too few parameter [Number of shifts]"  '
    $endif                                         '
    $if pDelayMS = 0                               '
    $error "Too few parameter [Number of MS]"      '
    $endif                                         '
    PRODL = pNumberShift                           '
    RotateDelay = pDelayMS                         '
    _mRotateRightDispArray

_mRotateRightDispArray Macro-
    GoSub RotateRightDispArray_Sub
Endm

#ifMacro- _mRotateRightDispArray
RotateRightDispArray_Sub:
    Repeat 
        #ifdef watchdog_req
        Clrwdt
        #endif
        GoSub RotateRightDigit_Sub  '/ Shift the array to the Right.
        DelayMS RotateDelay         '/ Delay between rotations.
        Dec PRODL
    Until PRODL = 0                 '/ Shift all the digits.
    Return

RotateRightDigit_Sub:
'    DispLenArray = Len (DISP_ARRAY) '/ Accordinly with the number of digits.
    DispLenArray = LenghtDISP_ARRAY
    FSR0 = VarPtr (DISP_ARRAY)      '/ Start address of the array
    FSR1 = VarPtr (DISP_ARRAYcop)   '/ Start address of the array
    PRODH = INDF0                   '/ Save the first original byte.
    Inc FSR0                        '/ Shift pointer in origen + 1.
    Repeat
        #ifdef watchdog_req
        Clrwdt
        #endif
        POSTINC1 = POSTINC0         '/ Copy all bytes
        Dec DispLenArray         
    Until DispLenArray = 0
    Dec FSR1
    INDF1 = PRODH                   '/ Place the first byte            
    DisPlayInterruptEnable = 0 
    DISP_ARRAY = DISP_ARRAYcop      '/ Return the array to print.
    DisPlayInterruptEnable = 1 
    Return
#endIfMacro-    
'/------------------------------------------------------------------------------
$define RotateLeftDisplay(pNumberShift, pDelayMS) '
    $if NumberDigits < 3                          '
    $error "Too few digits for shifting"          '
    $endif                                        '
    $if pNumberShift = 0                          '
    $error "Too few parameter [Number of shifts]" '
    $endif                                        '
    $if pDelayMS = 0                              '
    $error "Too few parameter [Number of MS]"     '
    $endif                                        '
    PRODL = pNumberShift                          '
    RotateDelay = pDelayMS                        '
    _mRotateLeftDispArray

_mRotateLeftDispArray Macro-
    GoSub RotateLeftDispArray_Sub
Endm

#ifMacro- _mRotateLeftDispArray
RotateLeftDispArray_Sub:
    Repeat 
        #ifdef watchdog_req
        Clrwdt
        #endif
        GoSub RotateLeftDigit_Sub   '/ Shift the array to the Left.
        DelayMS RotateDelay         '/ Delay between rotations.
        Dec PRODL
    Until PRODL = 0                 '/ Shift all the digits.
    Return

RotateLeftDigit_Sub:
'    DispLenArray = Len (DISP_ARRAY) '/ Accordinly with the number of digits.
    DispLenArray = LenghtDISP_ARRAY
    FSR0 = VarPtr (DISP_ARRAY)      '/ Start address of the array
    FSR1 = VarPtr (DISP_ARRAYcop)   '/ Start address of the array
    Dec DispLenArray                '/ To sum FSR0/FSR1 correctly.
    FSR0 = FSR0 + DispLenArray      '/ End of address of the array
    FSR1 = FSR1 + DispLenArray      '/ End of address of the array
    Inc DispLenArray                '/ Restore the lenght of array.
    PRODH = INDF0                   '/ Save the last original byte.
    Dec FSR0                        '/ Shift pointer to final - 1.
    Repeat
        #ifdef watchdog_req
        Clrwdt
        #endif
        POSTDEC1 = POSTDEC0         '/ Copy all bytes
        Dec DispLenArray         
    Until DispLenArray = 0
    Inc FSR1
    INDF1 = PRODH                   '/ Place the last byte
    DisPlayInterruptEnable = 0 
    DISP_ARRAY = DISP_ARRAYcop      '/ Return the array To Print.
    DisPlayInterruptEnable = 1      
    Return
#endIfMacro-    
$endif
'===============================================================================
_MultiplexDigitsExit:
'===============================================================================
    DisplayInit()       '/ Initialize the Displays.
'===============================================================================
$endif  ' _MultiplexDigits
'===============================================================================


            
    
