'****************************************************************
'*  Name    : MulitplexB8DigitsNeg_Lib.inc                      *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2018 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 22/06/2018                                        *
'*  Version : 1.0                                               *
'*  Notes   : Proton Basic Compiler V3.6.0.3                    *
'*          : Original Code written by Les Johnson              *
'* Purpose  : Multiplexing 8 digits LED display with 2 PORTS    *
'*          : Using negative numbers                            *
'****************************************************************
'/ 28/06/2018 version 2.0
'/ Add a macro that allows to input every byte of your main program.
'/ The type of the variable is checked and makes an error at compiling if 
'/ the type of variable is diffrent as library.
'/

$ifndef _MultiplexDigits
$define _MultiplexDigits

'/------------------------------------------------------------------------------
    $if _device != _18F452
    $error "This library is for the PIC18F452 device only. Please write a new library"
    $endif

    $ifdef _ProteusTest
        $if _xtal != 8
        $error "It is better to use the Declare Xtal = 8 for Proteus"
        $endif   
    $endif    
'===============================================================================
'/ Configuration of the Config Fuses
    $define PLL_ConfigFuses OFF
    '$define _CLKOUT_Function_
    
'/------------------------------------------------------------------------------
'/ Configuration of the displays hardware.

    '/ Declare Display Mode (0 => Common Cathode) (1 => Common Anode) with transistors
    $define DisplayMode 0    

    '/ Declare Number of Digits (2,3,4,5,6,7,8)
    $define NumberDigits 8

    '/ Declare DecimalP position (0,1,2,3,4,5,6) (0 = No DP)
    $define DecimalP 0

    '/ Declare Shifting Digits option. (YES/NO)
    $define ShiftingDigits YES

'===============================================================================
'/ Declare the Timer used for scanning. (one only)
'    $define ScanTimer0
    $define ScanTimer2

'/ Declare the interruption flags (automatic).
    $ifdef ScanTimer0
    $define DisPlayInterruptFlag INTCONBits_TMR0IF
    $define DisPlayInterruptEnable INTCONBits_TMR0IE
    $endif

    $ifdef ScanTimer2    
    $define DisPlayInterruptFlag PIR1Bits_TMR2IF
    $define DisPlayInterruptEnable PIE1Bits_TMR2IE
    $endif

'/ Declare the HEX number option. (YES/NO)
    $define _HEXnumbers YES  

'/ Declare Hardware type (8 displays with 2 PORTS)
    $define _Hardware 0 
'===============================================================================
'/ Calculate the better option for Table Data (automaticly).
    '/ Declare Table Data (0 from ROM, 1 from eeprom)
    $if _eeprom > 0
    $define TableData 1    
    $else
    $define TableData 0    
    $endif

'/ Declare the outputs of scanning.
    $if DisplayMode = 0
        $define TurnOffDisp 0   ' Turn Off For transistor NPN    
        $define TurnOnDisp 1    ' Turn On For transistor NPN
    $else
        $define TurnOffDisp 1   ' Turn Off For transistor PNP    
        $define TurnOnDisp 0    ' Turn On For transistor PNP  
    $endif 

'/ Checking security
    $if NumberDigits = 8
        $if DecimalP > 6
        $error "No DP enabled For this Digit number"
        $endif
    $endif
    $if NumberDigits = 7
        $if DecimalP > 5
        $error "No DP enabled For this Digit number"
        $endif
    $endif
    $if NumberDigits = 6
        $if DecimalP > 4
        $error "No DP enabled For this Digit number"
        $endif
    $endif 
    $if NumberDigits = 5
        $if DecimalP > 3
        $error "No DP enabled For this Digit number"
        $endif
    $endif   
    $if NumberDigits = 4
        $if DecimalP > 2
        $error "No DP enabled For this Digit number"
        $endif
    $endif 
    $if NumberDigits = 3
        $if DecimalP > 1
        $error "No DP enabled For this Digit number"
        $endif
    $endif 
    $if NumberDigits = 2
        $if DecimalP > 0
        $error "No DP enabled For this Digit number"
        $endif
    $endif    
        
    $if NumberDigits < 2 Or NumberDigits > 8
    $error "NO such digit number allowed"
    $endif
    $if DisplayMode > 1
    $error "The DisplayMode must be 0 or 1"
    $endif
    $if TableData > 1
    $error "The Table of Data must be 0 = ROM or 1 = EEPROM"
    $endif

'/ Check a timer
    $if !_defined(ScanTimer0) And !_defined(ScanTimer2)
    $error "Timer0 or Timer2 must be declared"    
    $endif    
'===============================================================================
    GoTo _MultiplexDigitsExit
'===============================================================================
'/**** Added by Fuse Configurator ****
'/ Use the Fuse Configurator plug-in to change these settings

Config_Start
$if PLL_ConfigFuses = On
    $if (_xtal >= 16) And (_xtal <= 40)
    OSC = HSPLL	;HS oscillator, PLL enabled (Clock Frequency = 4 x FOSC)(Xtal piece 2,5-10 MHz)
    $else
        $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = On"
    $endif
$endif
$if PLL_ConfigFuses = OFF
    $if (_xtal >= 4) And (_xtal <= 25)
    OSC = HS	    ;HS oscillator PLL disabled  (medium power 4-20 MHz) (Xtal piece 10-32 MHz)
    $else
        $if (_xtal > 25) And (_xtal <= 40)
            $ifdef _CLKOUT_Function_
            OSC = EC     ; Exterior EC oscillator, CLKOUT function On RA6 
            $else
            OSC = ECIO   ; Exterior EC oscillator, port function on RA6 
            $endif
        $else
            $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = OFF"
        $endif
   $endif
$endif
    OSCS = OFF	;Oscillator system clock switch option is disabled (main oscillator is source)
    PWRT = On	;PWRT enabled
    BOR = On	;Brown-out Reset enabled
    BORV = 27	;VBOR set to 2.7V
    WDT = OFF	;WDT disabled (control is placed on the SWDTEN bit)
    WDTPS = 128	;1:128
    CCP2MUX = On	;CCP2 input/output is multiplexed with RC1
    STVR = OFF	;Stack Full/Underflow will not cause RESET
    LVP = OFF	;Low Voltage ICSP disabled
    Debug = OFF	;Background Debugger disabled. RB6 and RB7 configured as general purpose I/O pins.
    Cp0 = OFF	;Block 0 (000200-001FFFh) not code protected
    CP1 = OFF	;Block 1 (002000-003FFFh) not code protected
    CP2 = OFF	;Block 2 (004000-005FFFh) not code protected
    CP3 = OFF	;Block 3 (006000-007FFFh) not code protected
    CPB = OFF	;Boot Block (000000-0001FFh) not code protected
    CPD = OFF	;Data EEPROM not code protected
    WRT0 = OFF	;Block 0 (000200-001FFFh) not write protected
    WRT1 = OFF	;Block 1 (002000-003FFFh) not write protected
    WRT2 = OFF	;Block 2 (004000-005FFFh) not write protected
    WRT3 = OFF	;Block 3 (006000-007FFFh) not write protected
    WRTC = OFF	;Configuration registers (300000-3000FFh) not write protected
    WRTB = OFF	;Boot Block (000000-0001FFh) not write protected
    WRTD = OFF	;Data EEPROM not write protected
    EBTR0 = OFF	;Block 0 (000200-001FFFh) not protected from Table Reads executed in other blocks
    EBTR1 = OFF	;Block 1 (002000-003FFFh) not protected from Table Reads executed in other blocks
    EBTR2 = OFF	;Block 2 (004000-005FFFh) not protected from Table Reads executed in other blocks
    EBTR3 = OFF	;Block 3 (006000-007FFFh) not protected from Table Reads executed in other blocks
    EBTRB = OFF	;Boot Block (000000-0001FFh) not protected from Table Reads executed in other blocks
Config_End
'/**** End of Fuse Configurator Settings ****
'===============================================================================
    On_Hardware_Interrupt GoTo MULTILEX_DIGITS		' Point to the interrupt handler and ENABLE global interrupts
'===============================================================================       

    '/ Using system variables the routines could run faster.
    Dim DISPVAL_Flags As Byte System
    Dim F_NegativeNumber As DISPVAL_Flags.0
    
$if NumberDigits = 8
    Dim ONES As Byte System
    Dim TENS As Byte System
    Dim HUNDREDS As Byte System
    Dim THOUSANDS As Byte System
    Dim TENTHOUSANDS As Byte System
    Dim HUNDREDTHOUSANDS As Byte System
    Dim THOUSANDTHOUSANDS As Byte System
    Dim TENTHDTHOUSANDS As Byte System

    $if ShiftingDigits = YES
    Symbol LenghtDISP_ARRAY = 8
    Dim DISP_ARRAY As String * LenghtDISP_ARRAY 
    Dim PRINT_ONES As DISP_ARRAY#0              '/ Variable for interrupt routine.
    Dim PRINT_TENS As DISP_ARRAY#1              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As DISP_ARRAY#2             '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As DISP_ARRAY#3         '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As DISP_ARRAY#4      '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDTHOUSANDS As DISP_ARRAY#5  '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDTHOUSANDS As DISP_ARRAY#6 '/ Variable for interrupt routine.
    Dim PRINT_TENTHDTHOUSANDS As DISP_ARRAY#7   '/ Variable for interrupt routine.
    Dim DISP_ARRAYcop As String * LenghtDISP_ARRAY 
    $endif
    
    $if ShiftingDigits = NO
    Dim PRINT_ONES As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System         '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As Byte System      '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDTHOUSANDS As Byte System  '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDTHOUSANDS As Byte System '/ Variable for interrupt routine.
    Dim PRINT_TENTHDTHOUSANDS As Byte System   '/ Variable for interrupt routine.
    $endif    

    Dim DISPVAL	As SDword System                '/ The number to display on the LEDS
    Dim DISPVALinteger As Dword System          '/ Variable For Interrupt routine. The number to display on the LEDS
$endif

$if NumberDigits = 7
    Dim ONES As Byte System
    Dim TENS As Byte System
    Dim HUNDREDS As Byte System
    Dim THOUSANDS As Byte System
    Dim TENTHOUSANDS As Byte System
    Dim HUNDREDTHOUSANDS As Byte System
    Dim THOUSANDTHOUSANDS As Byte System

    $if ShiftingDigits = YES
    Symbol LenghtDISP_ARRAY = 7
    Dim DISP_ARRAY As String * LenghtDISP_ARRAY
    Dim PRINT_ONES As DISP_ARRAY#0              '/ Variable for interrupt routine.
    Dim PRINT_TENS As DISP_ARRAY#1              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As DISP_ARRAY#2          '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As DISP_ARRAY#3         '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As DISP_ARRAY#4      '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDTHOUSANDS As DISP_ARRAY#5  '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDTHOUSANDS As DISP_ARRAY#6 '/ Variable for interrupt routine.
    Dim DISP_ARRAYcop As String * LenghtDISP_ARRAY 
    $endif
    
    $if ShiftingDigits = NO
    Dim PRINT_ONES As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System         '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As Byte System      '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDTHOUSANDS As Byte System  '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDTHOUSANDS As Byte System '/ Variable for interrupt routine.
    $endif    
    
    Dim DISPVAL	As SDword System                '/ The number to display on the LEDS
    Dim DISPVALinteger As Dword System          '/ Variable For Interrupt routine. The number to display on the LEDS
$endif

$if NumberDigits = 6
    Dim ONES As Byte System
    Dim TENS As Byte System
    Dim HUNDREDS As Byte System
    Dim THOUSANDS As Byte System
    Dim TENTHOUSANDS As Byte System
    Dim HUNDREDTHOUSANDS As Byte System

    $if ShiftingDigits = YES
    Symbol LenghtDISP_ARRAY = 6
    Dim DISP_ARRAY As String * LenghtDISP_ARRAY 
    Dim PRINT_ONES As DISP_ARRAY#0              '/ Variable for interrupt routine.
    Dim PRINT_TENS As DISP_ARRAY#1              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As DISP_ARRAY#2          '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As DISP_ARRAY#3         '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As DISP_ARRAY#4      '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDTHOUSANDS As DISP_ARRAY#5  '/ Variable for interrupt routine.
    Dim DISP_ARRAYcop As String * LenghtDISP_ARRAY
    $endif

    $if ShiftingDigits = NO
    Dim PRINT_ONES As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System         '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As Byte System      '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDTHOUSANDS As Byte System  '/ Variable for interrupt routine.
    $endif

    Dim DISPVAL	As SDword System                '/ The number to display on the LEDS
    Dim DISPVALinteger As Dword System          '/ Variable For Interrupt routine. The number to display on the LEDS
$endif

$if NumberDigits = 5
    Dim ONES As Byte System
    Dim TENS As Byte System
    Dim HUNDREDS As Byte System
    Dim THOUSANDS As Byte System
    Dim TENTHOUSANDS As Byte System

    $if ShiftingDigits = YES
    Symbol LenghtDISP_ARRAY = 5
    Dim DISP_ARRAY As String * LenghtDISP_ARRAY 
    Dim PRINT_ONES As DISP_ARRAY#0              '/ Variable for interrupt routine.
    Dim PRINT_TENS As DISP_ARRAY#1              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As DISP_ARRAY#2          '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As DISP_ARRAY#3         '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As DISP_ARRAY#4      '/ Variable for interrupt routine.
    Dim DISP_ARRAYcop As String * LenghtDISP_ARRAY
    $endif
    
    $if ShiftingDigits = NO
    Dim PRINT_ONES As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System         '/ Variable for interrupt routine.
    Dim PRINT_TENTHOUSANDS As Byte System      '/ Variable for interrupt routine.
    $endif

    Dim DISPVAL	As SWord System             '/ The number to display on the LEDS
    Dim DISPVALinteger As Word System       '/ Variable For Interrupt routine. The number to display on the LEDS
$endif

$if NumberDigits = 4
    Dim ONES As Byte System
    Dim TENS As Byte System
    Dim HUNDREDS As Byte System
    Dim THOUSANDS As Byte System

    $if ShiftingDigits = YES
    Symbol LenghtDISP_ARRAY = 4
    Dim DISP_ARRAY As String * LenghtDISP_ARRAY 
    Dim PRINT_ONES As DISP_ARRAY#0              '/ Variable for interrupt routine.
    Dim PRINT_TENS As DISP_ARRAY#1              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As DISP_ARRAY#2          '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As DISP_ARRAY#3         '/ Variable for interrupt routine.
    Dim DISP_ARRAYcop As String * LenghtDISP_ARRAY
    $endif

    $if ShiftingDigits = NO
    Dim PRINT_ONES As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System              '/ Variable for interrupt routine.
    Dim PRINT_HUNDREDS As Byte System          '/ Variable for interrupt routine.
    Dim PRINT_THOUSANDS As Byte System         '/ Variable for interrupt routine.
    $endif

    Dim DISPVAL	As SWord System             '/ The number to display on the LEDS
    Dim DISPVALinteger As Word System       '/ Variable For Interrupt routine. The number to display on the LEDS
$endif
    
$if NumberDigits = 3
    '/ NO shift for 3 digits.
    Dim ONES As Byte System
    Dim TENS As Byte System
    Dim HUNDREDS As Byte System
    Dim PRINT_ONES As Byte System           '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System
    Dim PRINT_HUNDREDS As Byte System
    Dim DISPVAL	As SByte System             '/ The number to display on the LEDS
    Dim DISPVALinteger As Byte System       '/ Variable For Interrupt routine. The number to display on the LEDS
$endif
    
$if NumberDigits = 2
    '/ NO shift for 2 digits.
    Dim ONES As Byte System
    Dim TENS As Byte System
    Dim PRINT_ONES As Byte System           '/ Variable for interrupt routine.
    Dim PRINT_TENS As Byte System
    Dim DISPVAL	As SByte System             '/ The number to display on the LEDS
    Dim DISPVALinteger As Byte System       '/ Variable For Interrupt routine. The number to display on the LEDS
$endif
        
    $if NumberDigits = 8
    Dim TIMESHARE As Word System	    '/ Variable for interrupt routine. Used by the interrupt for time sharing
    $else
    Dim TIMESHARE As Byte System	    '/ Variable For Interrupt routine. Used by the interrupt for time sharing    
    $endif

$ifndef FSR0_Var
    $define FSR0_Var
	Dim FSR0 As FSR0L.Word         'Combine FSR0L\H into a single 16-bit register
$endif
$ifndef FSR1_Var
    $define FSR1_Var
	Dim FSR1 As FSR1L.Word         'Combine FSR0L\H into a single 16-bit register
$endif
'/------------------------------------------------------------------------------

    '/ Definition of the PORTS for Display segments.
    Symbol SEGMENTS = LATD
    Symbol PORT_Digits = LATB

    '/ Definition of the pins to enable displays to write.
    $if NumberDigits = 8
    Symbol DIGIT1 = LATB.0
    Symbol DIGIT2 = LATB.1
    Symbol DIGIT3 = LATB.2
    Symbol DIGIT4 = LATB.3 
    Symbol DIGIT5 = LATB.4
    Symbol DIGIT6 = LATB.5
    Symbol DIGIT7 = LATB.6
    Symbol DIGIT8 = LATB.7
    $endif

    $if NumberDigits = 7
    Symbol DIGIT1 = LATB.0
    Symbol DIGIT2 = LATB.1
    Symbol DIGIT3 = LATB.2
    Symbol DIGIT4 = LATB.3 
    Symbol DIGIT5 = LATB.4
    Symbol DIGIT6 = LATB.5
    Symbol DIGIT7 = LATB.6
    $endif

    $if NumberDigits = 6
    Symbol DIGIT1 = LATB.0
    Symbol DIGIT2 = LATB.1
    Symbol DIGIT3 = LATB.2
    Symbol DIGIT4 = LATB.3 
    Symbol DIGIT5 = LATB.4
    Symbol DIGIT6 = LATB.5
    $endif

    $if NumberDigits = 5
    Symbol DIGIT1 = LATB.0
    Symbol DIGIT2 = LATB.1
    Symbol DIGIT3 = LATB.2
    Symbol DIGIT4 = LATB.3 
    Symbol DIGIT5 = LATB.4
    $endif

    $if NumberDigits = 4
    Symbol DIGIT1 = LATB.0
    Symbol DIGIT2 = LATB.1
    Symbol DIGIT3 = LATB.2
    Symbol DIGIT4 = LATB.3 
    $endif
    
    $if NumberDigits = 3
    Symbol DIGIT1 = LATB.0
    Symbol DIGIT2 = LATB.1
    Symbol DIGIT3 = LATB.2
    $endif
    
    $if NumberDigits = 2
    Symbol DIGIT1 = LATB.0
    Symbol DIGIT2 = LATB.1
    $endif            
'===============================================================================
'/ Seven Segment data written for Commom Anode display.
'/ Using some tricks it is not necessary using another table for common cathode.
$if TableData = 0                                               ' A   B   C   D   E   F
SegmentsData: CData Byte 192,249,164,176,153,146,131,248,128,152,136,131,198,161,134,142,255

'/           0          1          2          3          4          5          
'/Segments: %11000000, %11111001, %10100100, %10110000, %10011001, %10010010,_
'/  6          7          8          9          255   
'/ %10000011, %11111000, %10000000, %10011000, %11111111 

'===============================================================================
$else
'/ For more efficiency the Table EData must be placed in address 0 of the eeprom.
SegmentsData EData Byte 192,249,164,176,153,146,131,248,128,152,136,131,198,161,134,142,255
$endif
'===============================================================================
'===============================================================================
'/ INTERRUPT HANDLER 
'/ Multiplexing the 8-digits
'/
MULTILEX_DIGITS:
Context Save			
    If DisPlayInterruptFlag = 1 Then 		
        '/ Process the first display
        If TIMESHARE.0 = 1 Then             '/ If it's our turn then...
            $if NumberDigits = 8
            DIGIT8 = TurnOffDisp            '/ Turn OFF the fifht LED
            $endif
            $if NumberDigits = 7
            DIGIT7 = TurnOffDisp            '/ Turn OFF the fifht LED
            $endif
            $if NumberDigits = 6
            DIGIT6 = TurnOffDisp            '/ Turn OFF the fifht LED
            $endif
            $if NumberDigits = 5
            DIGIT5 = TurnOffDisp            '/ Turn OFF the fifht LED
            $endif
            $if NumberDigits = 4
            DIGIT4 = TurnOffDisp            '/ Turn OFF the fourth LED
            $endif
            $if NumberDigits = 3
            DIGIT3 = TurnOffDisp            '/ Turn OFF the third LED
            $endif                
            $if NumberDigits = 2
            DIGIT2 = TurnOffDisp            '/ Turn OFF the second LED
            $endif  

            $if DisplayMode = 0
            SEGMENTS = ~PRINT_ONES          '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            SEGMENTS = PRINT_ONES           '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            DelayCS 2                       '/ Delay to stabilize data. Adjust the value for real PIC.
            DIGIT1 = TurnOnDisp             '/ Turn ON the first LED
   
        '/ Process the second display									
        ElseIf TIMESHARE.1 = 1 Then         '/ If it's our turn then...
            DIGIT1 = TurnOffDisp            '/ Turn OFF the first LED

            $if DisplayMode = 0
            SEGMENTS = ~PRINT_TENS          '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            SEGMENTS = PRINT_TENS           '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            DelayCS 2                       '/ Delay to stabilize data. Adjust the value for real PIC.
            DIGIT2 = TurnOnDisp             '/ Turn On the second LED

        '/ Process the third display									
        $if NumberDigits > 2
        ElseIf TIMESHARE.2 = 1 Then         '/ If it's our turn then...
            DIGIT2 = TurnOffDisp            '/ Turn OFF the second LED

            $if DisplayMode = 0
            SEGMENTS = ~PRINT_HUNDREDS      '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            SEGMENTS = PRINT_HUNDREDS       '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            DelayCS 2                       '/ Delay to stabilize data. Adjust the value for real PIC.
            DIGIT3 = TurnOnDisp             '/ Turn On the third LED
        $endif
                
        '/ Process the fourth display			
        $if NumberDigits > 3
        ElseIf TIMESHARE.3 = 1 Then         '/ If it's our turn then...
            DIGIT3 = TurnOffDisp            '/ Turn OFF the third LED

            $if DisplayMode = 0
            SEGMENTS = ~PRINT_THOUSANDS     '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            SEGMENTS = PRINT_THOUSANDS      '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            DelayCS 2                       '/ Delay to stabilize data. Adjust the value for real PIC.
            DIGIT4 = TurnOnDisp             '/ Turn On the fourth LED
        $endif

        '/ Process the fifht display			
        $if NumberDigits > 4
        ElseIf TIMESHARE.4 = 1 Then         '/ If it's our turn then...
            DIGIT4 = TurnOffDisp            '/ Turn OFF the fifht LED

            $if DisplayMode = 0
            SEGMENTS = ~PRINT_TENTHOUSANDS  '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            SEGMENTS = PRINT_TENTHOUSANDS   '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            DelayCS 2                       '/ Delay to stabilize data. Adjust the value for real PIC.
            DIGIT5 = TurnOnDisp             '/ Turn On the fourth LED
        $endif

        '/ Process the sixfht display			
        $if NumberDigits > 5
        ElseIf TIMESHARE.5 = 1 Then             '/ If it's our turn then...
            DIGIT5 = TurnOffDisp                '/ Turn OFF the sixfht LED

            $if DisplayMode = 0
            SEGMENTS = ~PRINT_HUNDREDTHOUSANDS  '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            SEGMENTS = PRINT_HUNDREDTHOUSANDS   '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            DelayCS 2                           '/ Delay to stabilize data. Adjust the value for real PIC.
            DIGIT6 = TurnOnDisp                 '/ Turn On the fourth LED
        $endif

        '/ Process the sevenfht display			
        $if NumberDigits > 6
        ElseIf TIMESHARE.6 = 1 Then             '/ If it's our turn then...
            DIGIT6 = TurnOffDisp                '/ Turn OFF the sixfht LED

            $if DisplayMode = 0
            SEGMENTS = ~PRINT_THOUSANDTHOUSANDS '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            SEGMENTS = PRINT_THOUSANDTHOUSANDS  '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            DelayCS 2                           '/ Delay to stabilize data. Adjust the value for real PIC.
            DIGIT7 = TurnOnDisp                 '/ Turn On the fourth LED
        $endif

        '/ Process the eightfht display			
        $if NumberDigits > 7
        ElseIf TIMESHARE.7 = 1 Then             '/ If it's our turn then...
            DIGIT7 = TurnOffDisp                '/ Turn OFF the sixfht LED

            $if DisplayMode = 0
            SEGMENTS = ~PRINT_TENTHDTHOUSANDS   '/ For Commom Cathode display / Set up the pattern for PORTD
            $else
            SEGMENTS = PRINT_TENTHDTHOUSANDS    '/ For Commom Anode display / Set up the pattern for PORTD            
            $endif
            DelayCS 2                           '/ Delay to stabilize data. Adjust the value for real PIC.
            DIGIT8 = TurnOnDisp                 '/ Turn On the fourth LED
        $endif
        EndIf    

TIMESHARING:
        '/ Process the scanning.
        TIMESHARE = TIMESHARE << 1      '/ Shift bit to set the next digit display.

        '/ Checking the last digit & reset scanning.        
        $if NumberDigits = 8
        If TIMESHARE.8 = 1 Then			'/ Is Bit 8 Set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif

        $if NumberDigits = 7
        If TIMESHARE.7 = 1 Then			'/ Is bit 7 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 6
        If TIMESHARE.6 = 1 Then			'/ Is bit 6 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 5
        If TIMESHARE.5 = 1 Then			'/ Is bit 5 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 4
        If TIMESHARE.4 = 1 Then			'/ Is bit 4 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        $if NumberDigits = 3
        If TIMESHARE.3 = 1 Then			'/ Is bit 3 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif        
        $if NumberDigits = 2
        If TIMESHARE.2 = 1 Then			'/ Is bit 2 set ?
            TIMESHARE = 1               '/ Yes. So Set TIMESHARE To 1
        EndIf
        $endif
        DisPlayInterruptFlag = 0        '/ Reset TMR0 interrupt flag
    EndIf
Context Restore			                '/ Exit the interrupt routine		

'===============================================================================
'===============================================================================
'    Include "D:/MyIncFiles/MyArraysLib.Inc"
'===============================================================================		
'/ Build up the seperate digits of variable DISPVAL
$define PrintDigits() GoSub BuildDigits_End

$define BuildDigits(pValue) '
    DISPVAL = pValue        '
    _mBuildDigits pValue

'/ This macro is used for testing variables only.
_mBuildDigits Macro- pValue
    #if (Prm_Count != 1)
        #error "_mBuildDigits - This macro uses 1 parameter!"
    #else
        #if (Prm_1 != SDword) && (Prm_1 != SWord) && (Prm_1 != SByte)
            #error "_mPrintDispInt Macro - (Param1) Should be a SDword, SWord or SByte Variable"
        #endif
    #endif
    GoSub Build_Digits_Sub
Endm

Build_Digits_Sub:
    #ifdef watchdog_req
    Clrwdt
    #endif
    F_NegativeNumber = 0
    $if NumberDigits = 2
    '/ DISPVAL = Byte 
    If DISPVAL.7 = 1 Then
        F_NegativeNumber = 1
    EndIf
    $endif
    $if NumberDigits = 3
    '/ DISPVAL = Byte 
    If DISPVAL.7 = 1 Then
        F_NegativeNumber = 1
    EndIf
    $endif
    $if NumberDigits = 4
    '/ DISPVAL = Word 
    If DISPVAL.15 = 1 Then
        F_NegativeNumber = 1
    EndIf
    $endif
    $if NumberDigits = 5
    '/ DISPVAL = Word 
    If DISPVAL.15 = 1 Then
        F_NegativeNumber = 1
    EndIf
    $endif    
    $if NumberDigits = 6
    '/ DISPVAL = Dword 
    If DISPVAL.31 = 1 Then
        F_NegativeNumber = 1
    EndIf
    $endif     
    $if NumberDigits = 7
    '/ DISPVAL = Dword 
    If DISPVAL.31 = 1 Then
        F_NegativeNumber = 1
    EndIf
    $endif 
    $if NumberDigits = 8
    '/ DISPVAL = Dword 
    If DISPVAL.31 = 1 Then
        F_NegativeNumber = 1
    EndIf
    $endif

    '/ Check the negative number.
    If F_NegativeNumber = 1 Then
        DISPVALinteger = Abs(DISPVAL)    
    Else
        DISPVALinteger = DISPVAL
    EndIf

    $if TableData = 1
    EECON1 = 0          ' Table Data in eeprom. Clear only once to save lines of code.
    $endif

    '/ Built Digit ONES
    $if TableData = 0
    ' If the PIC has not an eeprom use this code.
    ' 14 lines of asm code for all function.
    WREG = Dig DISPVALinteger,0                ' Using WREG 1 line of code is saved.
    ONES = CRead8 SegmentsData[WREG]    ' Using WREG 1 line of code is saved.
    $else
    ' If the PIC has an eeprom better use this code.
    ' Only 9 lines of asm code for all function.
    EEADR = Dig DISPVALinteger,0               
    EECON1bits_RD = 1
    ONES = EEDATA
    $endif 

    '/ Built Digit TENS
    If DISPVALinteger < 10 Then
        $if NumberDigits = 8 
        If F_NegativeNumber = 1 Then
    	TENS = 191                 '/ Print the (-) symbol.        
        Else
    	TENS = 255                 '/ Blank the TENS DIGIT
        EndIf
    	HUNDREDS = 255			   '/ Blank the HUNDREDS DIGIT
    	THOUSANDS = 255			   '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 255	       '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 255     '/ Blank the TENTHOUSANDS DIGIT
    	THOUSANDTHOUSANDS = 255    '/ Blank the THOUSANDTHOUSANDS DIGIT
        TENTHDTHOUSANDS = 255      '/ Blank the TENTHDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 7 
        If F_NegativeNumber = 1 Then
    	TENS = 191                 '/ Print the (-) symbol.        
        Else
    	TENS = 255                 '/ Blank the TENS DIGIT
        EndIf
    	HUNDREDS = 255			   '/ Blank the HUNDREDS DIGIT
    	THOUSANDS = 255			   '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 255	       '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 255     '/ Blank the TENTHOUSANDS DIGIT
    	THOUSANDTHOUSANDS = 255    '/ Blank the THOUSANDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 6 
        If F_NegativeNumber = 1 Then
    	TENS = 191                 '/ Print the (-) Symbol.
        Else
    	TENS = 255                 '/ Blank the TENS DIGIT
        EndIf
    	HUNDREDS = 255			   '/ Blank the HUNDREDS DIGIT
    	THOUSANDS = 255			   '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 255	       '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 255     '/ Blank the TENTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 5 
        If F_NegativeNumber = 1 Then
    	TENS = 191                 '/ Print the (-) Symbol.
        Else
    	TENS = 255                 '/ Blank the TENS DIGIT
        EndIf
    	HUNDREDS = 255			   '/ Blank the HUNDREDS DIGIT
    	THOUSANDS = 255			   '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 255	       '/ Blank the TENTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 4 
        If F_NegativeNumber = 1 Then
    	TENS = 191                 '/ Print the (-) Symbol.
        Else
    	TENS = 255                 '/ Blank the TENS DIGIT
        EndIf
    	HUNDREDS = 255			   '/ Blank the HUNDREDS DIGIT
    	THOUSANDS = 255			   '/ Blank the THOUSANDS DIGIT
        $endif
        $if NumberDigits = 3 
        If F_NegativeNumber = 1 Then
    	TENS = 191                 '/ Print the (-) Symbol.
        Else
    	TENS = 255                 '/ Blank the TENS DIGIT
        EndIf
    	HUNDREDS = 255			   '/ Blank the HUNDREDS DIGIT
        $endif
        $if NumberDigits = 2 
        If F_NegativeNumber = 1 Then
    	TENS = 191                 '/ Print the (-) Symbol.
        Else
    	TENS = 255                 '/ Blank the TENS DIGIT
        EndIf
        $endif
        $if NumberDigits > 2
        GoTo BuildDigits_End          '/ No more digits to check.
        $endif
    Else
        $if NumberDigits = 2
        '/ Display overrun. Too much value to display: DISPVAL < -9 or DISPVAL > +9
        TENS = 134                 '/ (E) Print Error symbol.        
        $else
            $if TableData = 0
            WREG = Dig DISPVALinteger,1    ' Using WREG 1 line of code is saved.
            TENS = CRead8 SegmentsData[WREG]
            $else
            ' It is more faster using the eeprom.
            EEADR = Dig DISPVALinteger,1               
            EECON1bits_RD = 1
            TENS = EEDATA
            $endif 
        $endif
    EndIf
    
    '/ Built Digit HUNDREDS
    $if NumberDigits > 2 
    If DISPVALinteger < 100 Then
        $if NumberDigits = 8 
        If F_NegativeNumber = 1 Then
    	   HUNDREDS = 191          '/ Print the (-) Symbol.
        Else
    	   HUNDREDS = 255          '/ Blank the HUNDREDS DIGIT
        EndIf
    	THOUSANDS = 255			   '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 255	       '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 255     '/ Blank the HUNDREDTHOUSANDS DIGIT
    	THOUSANDTHOUSANDS = 255    '/ Blank the THOUSANDTHOUSANDS DIGIT
        TENTHDTHOUSANDS = 255      '/ Blank the TENTHDTHOUSANDS DIGIT
        $endif
        $if NumberDigits = 7 
        If F_NegativeNumber = 1 Then
    	   HUNDREDS = 191          '/ Print the (-) Symbol.
        Else
    	   HUNDREDS = 255          '/ Blank the HUNDREDS DIGIT
        EndIf
    	THOUSANDS = 255			   '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 255	       '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 255     '/ Blank the HUNDREDTHOUSANDS DIGIT
    	THOUSANDTHOUSANDS = 255    '/ Blank the THOUSANDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 6 
        If F_NegativeNumber = 1 Then
    	   HUNDREDS = 191          '/ Print the (-) Symbol.
        Else
    	   HUNDREDS = 255          '/ Blank the HUNDREDS DIGIT
        EndIf
    	THOUSANDS = 255			   '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 255	       '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 255     '/ Blank the HUNDREDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 5
        If F_NegativeNumber = 1 Then
    	   HUNDREDS = 191        '/ Print the (-) Symbol.
        Else
    	   HUNDREDS = 255        '/ Blank the HUNDREDS DIGIT
        EndIf
    	THOUSANDS = 255			 '/ Blank the THOUSANDS DIGIT
    	TENTHOUSANDS = 255	     '/ Blank the TENTHOUSANDS DIGIT
        $endif
        $if NumberDigits = 4
        If F_NegativeNumber = 1 Then
    	   HUNDREDS = 191        '/ Print the (-) Symbol.
        Else
    	   HUNDREDS = 255        '/ Blank the HUNDREDS DIGIT
        EndIf
    	THOUSANDS = 255			 '/ Blank the THOUSANDS DIGIT
        $endif
        $if NumberDigits = 3
        If F_NegativeNumber = 1 Then
    	   HUNDREDS = 191        '/ Print the (-) Symbol.
        Else
    	   HUNDREDS = 255        '/ Blank the HUNDREDS DIGIT
        EndIf
        $endif        
        $if NumberDigits > 3
        GoTo BuildDigits_End          '/ No more digits to check.
        $endif
    Else        	
        $if NumberDigits = 3
        '/ Display overrun. Too much value to display: DISPVAL < -99 or DISPVAL > +99
        HUNDREDS = 134          '/ (E) Print Error symbol.        
        $else
            $if TableData = 0
            WREG = Dig DISPVALinteger,2             ' Using WREG 1 line of code is saved.
            HUNDREDS = CRead8 SegmentsData[WREG]
            $else
            ' It is more faster using the eeprom.
            EEADR = Dig DISPVALinteger,2               
            EECON1bits_RD = 1
            HUNDREDS = EEDATA
            $endif 
        $endif
    EndIf
    $endif

    '/ Built Digit THOUSANDS
    $if NumberDigits > 3    
    If DISPVALinteger < 1000 Then
        $if NumberDigits = 8 
        If F_NegativeNumber = 1 Then
    	   THOUSANDS = 191         '/ Print the (-) Symbol.
        Else
    	   THOUSANDS = 255         '/ Blank the THOUSANDS DIGIT
        EndIf
    	TENTHOUSANDS = 255	       '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 255     '/ Blank the HUNDREDTHOUSANDS DIGIT
    	THOUSANDTHOUSANDS = 255    '/ Blank the THOUSANDTHOUSANDS DIGIT
        TENTHDTHOUSANDS = 255      '/ Blank the TENTHDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 7 
        If F_NegativeNumber = 1 Then
    	   THOUSANDS = 191         '/ Print the (-) Symbol.
        Else
    	   THOUSANDS = 255         '/ Blank the THOUSANDS DIGIT
        EndIf
    	TENTHOUSANDS = 255	       '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 255     '/ Blank the HUNDREDTHOUSANDS DIGIT
    	THOUSANDTHOUSANDS = 255    '/ Blank the THOUSANDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 6 
        If F_NegativeNumber = 1 Then
    	   THOUSANDS = 191         '/ Print the (-) Symbol.
        Else
    	   THOUSANDS = 255         '/ Blank the THOUSANDS DIGIT
        EndIf
    	TENTHOUSANDS = 255	       '/ Blank the TENTHOUSANDS DIGIT
    	HUNDREDTHOUSANDS = 255     '/ Blank the HUNDREDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 5
        If F_NegativeNumber = 1 Then
    	   THOUSANDS = 191       '/ Print the (-) Symbol. 
        Else
    	   THOUSANDS = 255       '/ Blank the THOUSANDS DIGIT
        EndIf
    	TENTHOUSANDS = 255	     '/ Blank the TENTHOUSANDS DIGIT
        $endif
        $if NumberDigits = 4
        If F_NegativeNumber = 1 Then
    	   THOUSANDS = 191       '/ Print the (-) Symbol. 
        Else
    	   THOUSANDS = 255       '/ Blank the THOUSANDS DIGIT
        EndIf
        $endif
        $if NumberDigits > 4
        GoTo BuildDigits_End          '/ No more digits to check.
        $endif
    Else        	
        $if NumberDigits = 4
        '/ Display overrun. Too much value to display: DISPVAL < -999 or DISPVAL > +999
        THOUSANDS = 134          '/ (E) Print Error symbol.        
        $else
            $if TableData = 0
            WREG = Dig DISPVALinteger,3    ' Using WREG 1 line of code is saved.
            THOUSANDS = CRead8 SegmentsData[WREG]
            $else
            ' It is more faster using the eeprom.
            EEADR = Dig DISPVALinteger,3               
            EECON1bits_RD = 1
            THOUSANDS = EEDATA
            $endif 
        $endif
    EndIf			
    $endif

    '/ Built Digit TENTHOUSANDS
    $if NumberDigits > 4   
    If DISPVALinteger < 10000 Then
        $if NumberDigits = 8 
        If F_NegativeNumber = 1 Then
    	   TENTHOUSANDS = 191          '/ Print the (-) Symbol.
        Else
    	   TENTHOUSANDS = 255          '/ Blank the TENTHOUSANDS DIGIT
        EndIf
    	HUNDREDTHOUSANDS = 255         '/ Blank the HUNDREDTHOUSANDS DIGIT
    	THOUSANDTHOUSANDS = 255        '/ Blank the THOUSANDTHOUSANDS DIGIT
        TENTHDTHOUSANDS = 255          '/ Blank the TENTHDTHOUSANDS DIGIT
        $endif
        $if NumberDigits = 7 
        If F_NegativeNumber = 1 Then
    	   TENTHOUSANDS = 191          '/ Print the (-) Symbol.
        Else
    	   TENTHOUSANDS = 255          '/ Blank the TENTHOUSANDS DIGIT
        EndIf
    	HUNDREDTHOUSANDS = 255         '/ Blank the HUNDREDTHOUSANDS DIGIT
    	THOUSANDTHOUSANDS = 255        '/ Blank the THOUSANDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 6 
        If F_NegativeNumber = 1 Then
    	   TENTHOUSANDS = 191         '/ Print the (-) Symbol.
        Else
    	   TENTHOUSANDS = 255         '/ Blank the TENTHOUSANDS DIGIT
        EndIf
    	HUNDREDTHOUSANDS = 255        '/ Blank the HUNDREDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 5
        If F_NegativeNumber = 1 Then
    	   TENTHOUSANDS = 191         '/ Print the (-) Symbol.
        Else
    	   TENTHOUSANDS = 255         '/ Blank the TENTHOUSANDS DIGIT
        EndIf
        $endif
        $if NumberDigits > 5
        GoTo BuildDigits_End          '/ No more digits to check.
        $endif
    Else        	
        $if NumberDigits = 5
        '/ Display overrun. Too much value to display: DISPVAL < -9999 or DISPVAL > +9999
        TENTHOUSANDS = 134            '/ (E) Print Error symbol.        
        $else
            $if TableData = 0
            WREG = Dig DISPVALinteger,4    ' Using WREG 1 line of code is saved.
            TENTHOUSANDS = CRead8 SegmentsData[WREG]
            $else
            ' It is more faster using the eeprom.
            EEADR = Dig DISPVALinteger,4               
            EECON1bits_RD = 1
            TENTHOUSANDS = EEDATA
            $endif
        $endif
    EndIf			
    $endif

    '/ Built Digit HUNDREDTHOUSANDS
    $if NumberDigits > 5   
    If DISPVALinteger < 100000 Then
        $if NumberDigits = 8 
        If F_NegativeNumber = 1 Then
    	   HUNDREDTHOUSANDS = 191      '/ Print the (-) Symbol.  
        Else
    	   HUNDREDTHOUSANDS = 255      '/ Blank the HUNDREDTHOUSANDS DIGIT
        EndIf
    	THOUSANDTHOUSANDS = 255        '/ Blank the THOUSANDTHOUSANDS DIGIT
        TENTHDTHOUSANDS = 255          '/ Blank the TENTHDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 7 
        If F_NegativeNumber = 1 Then
    	   HUNDREDTHOUSANDS = 191      '/ Print the (-) Symbol.  
        Else
    	   HUNDREDTHOUSANDS = 255      '/ Blank the HUNDREDTHOUSANDS DIGIT
        EndIf
    	THOUSANDTHOUSANDS = 255        '/ Blank the THOUSANDTHOUSANDS DIGIT
        $endif 
        $if NumberDigits = 6
        If F_NegativeNumber = 1 Then
    	   HUNDREDTHOUSANDS = 191        
        Else
    	   HUNDREDTHOUSANDS = 255     '/ Blank the HUNDREDTHOUSANDS DIGIT
        EndIf
        $endif
        $if NumberDigits > 6
        GoTo BuildDigits_End          '/ No more digits to check.
        $endif
    Else        	
        $if NumberDigits = 6
        '/ Display overrun. Too much value to display: DISPVAL < -99999 or DISPVAL > +99999
        HUNDREDTHOUSANDS = 134        '/ (E) Print Error symbol.        
        $else
            $if TableData = 0
            WREG = Dig DISPVALinteger,5                 ' Using WREG 1 line of code is saved.
            HUNDREDTHOUSANDS = CRead8 SegmentsData[WREG]
            $else
            ' It is more faster using the eeprom.
            EEADR = Dig DISPVALinteger,5               
            EECON1bits_RD = 1
            HUNDREDTHOUSANDS = EEDATA
            $endif
        $endif
    EndIf			
    $endif

    '/ Built Digit THOUSANDTHOUSANDS
    $if NumberDigits > 6   
    If DISPVALinteger < 1000000 Then
        $if NumberDigits = 8 
        If F_NegativeNumber = 1 Then
    	   THOUSANDTHOUSANDS = 191     '/ Print the (-) Symbol.        
        Else
    	   THOUSANDTHOUSANDS = 255     '/ Blank the THOUSANDTHOUSANDS DIGIT
        EndIf
        TENTHDTHOUSANDS = 255          '/ Blank the TENTHDTHOUSANDS DIGIT
        $endif
        $if NumberDigits = 7 
        If F_NegativeNumber = 1 Then
    	   THOUSANDTHOUSANDS = 191     '/ Print the (-) Symbol.        
        Else
    	   THOUSANDTHOUSANDS = 255     '/ Blank the THOUSANDTHOUSANDS DIGIT
        EndIf
        $endif 
        $if NumberDigits > 7
        GoTo BuildDigits_End          '/ No more digits to check.
        $endif
    Else        	
        $if NumberDigits = 7
        '/ Display overrun. Too much value to display: DISPVAL < -999999 or DISPVAL > +999999
        THOUSANDTHOUSANDS = 134       '/ (E) Print Error symbol.        
        $else
            $if TableData = 0
            WREG = Dig DISPVALinteger,6     ' Using WREG 1 line of code is saved.
            THOUSANDTHOUSANDS = CRead8 SegmentsData[WREG]
            $else
            ' It is more faster using the eeprom.
            EEADR = Dig DISPVALinteger,6               
            EECON1bits_RD = 1
            THOUSANDTHOUSANDS = EEDATA
            $endif        
        $endif
    EndIf			
    $endif

    '/ Built Digit TENTHDTHOUSANDS
    $if NumberDigits > 7   
    If DISPVALinteger < 10000000 Then
'        $if NumberDigits = 8 
        If F_NegativeNumber = 1 Then
    	   TENTHDTHOUSANDS = 191     '/ Print the (-) Symbol.        
        Else
    	   TENTHDTHOUSANDS = 255     '/ Blank the TENTHDTHOUSANDS DIGIT
        EndIf
'        $endif
    Else        	
'        $if TableData = 0
'        WREG = Dig DISPVALinteger,7                         ' Using WREG 1 line of code is saved.
'        TENTHDTHOUSANDS = CRead8 SegmentsData[WREG]
'        $else
'        ' It is more faster using the eeprom.
'        EEADR = Dig DISPVALinteger,7               
'        EECON1bits_RD = 1
'        TENTHDTHOUSANDS = EEDATA
'        $endif

        '/ Display overrun. Too much value to display: DISPVAL < -9999999 or DISPVAL > +9999999
        TENTHDTHOUSANDS = 134     '/ (E) Print Error symbol.
    EndIf			
    $endif

BuildDigits_End:
    #ifdef watchdog_req
    Clrwdt
    #endif
    '/ Prepare Display 8 digits.
    $if NumberDigits = 8
        '/ Set the DP.
        $if DecimalP = 1 
        TENS.7 = 0         
        $endif
        $if DecimalP = 2 
        HUNDREDS.7 = 0
        $endif
        $if DecimalP = 3 
        THOUSANDS.7 = 0
        $endif
        $if DecimalP = 4 
        TENTHOUSANDS.7 = 0
        $endif
        $if DecimalP = 5 
        HUNDREDTHOUSANDS.7 = 0
        $endif
        $if DecimalP = 6 
        THOUSANDTHOUSANDS.7 = 0
        $endif
               
        ' Make a copy to print the values for the interrupt routine.
        DisPlayInterruptEnable = 0           '/ Avoid blinking.
        PRINT_ONES = ONES
        PRINT_TENS = TENS
        PRINT_HUNDREDS = HUNDREDS
        PRINT_THOUSANDS = THOUSANDS        
        PRINT_TENTHOUSANDS = TENTHOUSANDS 
        PRINT_HUNDREDTHOUSANDS = HUNDREDTHOUSANDS 
        PRINT_THOUSANDTHOUSANDS = THOUSANDTHOUSANDS
        PRINT_TENTHDTHOUSANDS = TENTHDTHOUSANDS
        DisPlayInterruptEnable = 1
    $endif

    '/ Prepare Display 7 digits.
    $if NumberDigits = 7
        '/ Set the DP.
        $if DecimalP = 1 
        TENS.7 = 0         
        $endif
        $if DecimalP = 2 
        HUNDREDS.7 = 0
        $endif
        $if DecimalP = 3 
        THOUSANDS.7 = 0
        $endif
        $if DecimalP = 4 
        TENTHOUSANDS.7 = 0
        $endif
        $if DecimalP = 5 
        HUNDREDTHOUSANDS.7 = 0
        $endif
                
        ' Make a copy to print the values for the interrupt routine.
        DisPlayInterruptEnable = 0           '/ Avoid blinking.
        PRINT_ONES = ONES
        PRINT_TENS = TENS
        PRINT_HUNDREDS = HUNDREDS
        PRINT_THOUSANDS = THOUSANDS        
        PRINT_TENTHOUSANDS = TENTHOUSANDS 
        PRINT_HUNDREDTHOUSANDS = HUNDREDTHOUSANDS 
        PRINT_THOUSANDTHOUSANDS = THOUSANDTHOUSANDS
        DisPlayInterruptEnable = 1
    $endif

    '/ Prepare Display 6 digits.
    $if NumberDigits = 6
        '/ Set the DP.
        $if DecimalP = 1 
        TENS.7 = 0         
        $endif
        $if DecimalP = 2 
        HUNDREDS.7 = 0
        $endif
        $if DecimalP = 3 
        THOUSANDS.7 = 0
        $endif
        $if DecimalP = 4 
        TENTHOUSANDS.7 = 0
        $endif
        
        ' Make a copy to print the values for the interrupt routine.
        DisPlayInterruptEnable = 0           '/ Avoid blinking.
        PRINT_ONES = ONES
        PRINT_TENS = TENS
        PRINT_HUNDREDS = HUNDREDS
        PRINT_THOUSANDS = THOUSANDS        
        PRINT_TENTHOUSANDS = TENTHOUSANDS 
        PRINT_HUNDREDTHOUSANDS = HUNDREDTHOUSANDS 
        DisPlayInterruptEnable = 1
    $endif

    '/ Prepare Display 6 digits.
    $if NumberDigits = 5
        '/ Set the DP.
        $if DecimalP = 1 
        TENS.7 = 0         
        $endif
        $if DecimalP = 2 
        HUNDREDS.7 = 0
        $endif
        $if DecimalP = 3 
        THOUSANDS.7 = 0
        $endif

        ' Make a copy to print the values for the interrupt routine.
        DisPlayInterruptEnable = 0           '/ Avoid blinking.
        PRINT_ONES = ONES
        PRINT_TENS = TENS
        PRINT_HUNDREDS = HUNDREDS
        PRINT_THOUSANDS = THOUSANDS        
        PRINT_TENTHOUSANDS = TENTHOUSANDS 
        DisPlayInterruptEnable = 1
    $endif

    '/ Prepare Display 4 digits.
    $if NumberDigits = 4
        '/ Set the DP.
        $if DecimalP = 1 
        TENS.7 = 0         
        $endif
        $if DecimalP = 2 
        HUNDREDS.7 = 0
        $endif

        ' Make a copy to print the values for the interrupt routine.
        DisPlayInterruptEnable = 0           '/ Avoid blinking.
        PRINT_ONES = ONES
        PRINT_TENS = TENS
        PRINT_HUNDREDS = HUNDREDS
        PRINT_THOUSANDS = THOUSANDS        
        DisPlayInterruptEnable = 1
    $endif

    '/ Prepare Display 3 digits.
    $if NumberDigits = 3
        '/ Set the DP.
        $if DecimalP = 1 
        TENS.7 = 0         
        $endif

        ' Make a copy to print the values for the interrupt routine.
        DisPlayInterruptEnable = 0           '/ Avoid blinking.
        PRINT_ONES = ONES
        PRINT_TENS = TENS
        PRINT_HUNDREDS = HUNDREDS
        DisPlayInterruptEnable = 1
    $endif

    '/ Prepare Display 2 digits.
    $if NumberDigits = 2
        ' Make a copy to print the values for the interrupt routine.
        DisPlayInterruptEnable = 0           '/ Avoid blinking.
        PRINT_ONES = ONES
        PRINT_TENS = TENS
        DisPlayInterruptEnable = 1
    $endif
      
    Return    
'===============================================================================
'/ Subroutine: Init the displays
'/
$ifdef ScanTimer0
$define DisplayInit() GoSub DisplayInit_Sub

DisplayInit_Sub:
'/ CONFIGURATION OF TIMER0
'/ Set TMR0 to interrupt on rollover
'/ The TMR0H & TMR0l counters are not loaded to simplify the code. Then the timing
'/ could change in every Delare Xtal.
'/
    #ifdef watchdog_req
    Clrwdt
    #endif
    WDTCON = 0          '/ Without watchdog

    INTCON = 0                                              '/ Clear all interrupts
    While INTCONBits_GIE = 1 : INTCONBits_GIE = 0 : Wend	'/ Make sure global interrupts are off
    INTCONBits_TMR0IE = 0       '/ Disable the Timer0 interrupts.

    INTCON2Bits_RBPU = 1        '/ All PORTB pull-ups are disabled.
    RCONBits_IPEN = 1           '/ Enable priority levels on interrupts.
    INTCON2Bits_TMR0IP = 1      '/ Timer0 for High Interrupt.

    '/ Configure the Timer0 for FOSC = 4Mhz.
    $if _xtal = 4
    T0CON = %00000100           '/ div32 => 8,1 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    '/ Configure the Timer0 for FOSC = 8Mhz.
    $if NumberDigits = 8 Or NumberDigits = 7
        $if _xtal = 8
        T0CON = %00000100           '/ div32 => 4,0 mS Timer0 Prescaler Select bits 
        T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
        T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
        T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
        $endif
    $else
        $if _xtal = 8
        T0CON = %00000101           '/ div64 => 8,1 mS Timer0 Prescaler Select bits 
        T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
        T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
        T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
        $endif   
    $endif
    '/ Configure the Timer0 for FOSC = 16Mhz.
    $if _xtal = 16
    T0CON = %00000110           '/ div128 => 8,1 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    '/ Configure the Timer0 for FOSC = 20Mhz.
    $if _xtal = 20
    T0CON = %00000110           '/ div128 => 6,55 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    '/ Configure the Timer0 for FOSC = 32Mhz.
    $if _xtal = 32
    T0CON = %00000111           '/ div256 => 8,1 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 1        '/ 1 = Timer0 is configured as an 8-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    '/ Configure the Timer0 for FOSC = 40Mhz.
    $if _xtal = 40
    T0CON = %00000000           '/ div2 => 13,1 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 0        '/ 0 = Timer0 is configured as an 16-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    '/ Configure the Timer0 for FOSC = 64Mhz for Kxx PICs.
    $if _xtal = 64
    T0CON = %00000001           '/ div4 => 8,2 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 0        '/ 0 = Timer0 is configured as an 16-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    '/ Configure the Timer0 for FOSC = 80Mhz for K20 Pics.
    $if _xtal = 80
    T0CON = %00000000           '/ div2 => 6,55 mS Timer0 Prescaler Select bits 
    T0CONBits_T08BIT = 0        '/ 0 = Timer0 is configured as an 16-bit timer/counter  
    T0CONBits_T0CS = 0          '/ 0 = Internal instruction cycle clock (CLKO)
    T0CONBits_PSA = 0           '/ 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
    $endif

    TMR0H = 0                   '/ Clear the Timer0 counter.
    Nop
    TMR0L = 0
    Nop    
    T0CONBits_TMR0ON = 1        '/ Start the Timer0.
    '---------------------------------------------------------------------------
    '/ Set the PORTS & Pins to digital.
    ADCON1 = 7
	Output SEGMENTS             '/ Put the display PORT as Output.
	Input PORT_Digits           '/ Put the digit PORT as inputs.

    ' Write the lines OFF at reset depending the Display Mode.
    $if DisplayMode = 0
        $if NumberDigits = 8
        Low DIGIT1
        Low DIGIT2
        Low DIGIT3
        Low DIGIT4
        Low DIGIT5          
        Low DIGIT6
        Low DIGIT7
        Low DIGIT8
        $endif
        $if NumberDigits = 7
        Low DIGIT1
        Low DIGIT2
        Low DIGIT3
        Low DIGIT4
        Low DIGIT5          
        Low DIGIT6
        Low DIGIT7
        $endif
        $if NumberDigits = 6
        Low DIGIT1
        Low DIGIT2
        Low DIGIT3
        Low DIGIT4
        Low DIGIT5          
        Low DIGIT6
        $endif
        $if NumberDigits = 5
        Low DIGIT1
        Low DIGIT2
        Low DIGIT3
        Low DIGIT4
        Low DIGIT5          
        $endif
        $if NumberDigits = 4
        Low DIGIT1
        Low DIGIT2
        Low DIGIT3
        Low DIGIT4  
        $endif
        $if NumberDigits = 3
        Low DIGIT1
        Low DIGIT2
        Low DIGIT3
        $endif
        $if NumberDigits = 2
        Low DIGIT1
        Low DIGIT2
        $endif                
    $else
        $if NumberDigits = 8
        High DIGIT1
        High DIGIT2
        High DIGIT3
        High DIGIT4
        High DIGIT5          
        High DIGIT6
        High DIGIT7
        High DIGIT8
        $endif
        $if NumberDigits = 7
        High DIGIT1
        High DIGIT2
        High DIGIT3
        High DIGIT4
        High DIGIT5          
        High DIGIT6
        High DIGIT7
        $endif
        $if NumberDigits = 6
        High DIGIT1
        High DIGIT2
        High DIGIT3
        High DIGIT4
        High DIGIT5          
        High DIGIT6
        $endif
        $if NumberDigits = 5
        High DIGIT1
        High DIGIT2
        High DIGIT3
        High DIGIT4
        High DIGIT5          
        $endif
        $if NumberDigits = 4
        High DIGIT1
        High DIGIT2
        High DIGIT3
        High DIGIT4  
        $endif
        $if NumberDigits = 3
        High DIGIT1
        High DIGIT2
        High DIGIT3
        $endif
        $if NumberDigits = 2
        High DIGIT1
        High DIGIT2
        $endif        
    $endif
    
    ' Blanks directly the Port at reset depending of the option.
    $if DisplayMode = 0
    SEGMENTS = 0                '/ All displays cleared for common cathode.
    $else
    SEGMENTS = 255              '/ All displays cleared for common anode.
    $endif

	TIMESHARE = 1		        '/ Reset the time share variable

    '/ Blanks the Displays at reset.
    $if NumberDigits = 8
    PRINT_ONES = 255                '/ Blank the ONES DIGIT
    PRINT_TENS = 255                '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255            '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255           '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 255        '/ Blank the TENTHOUSANDS DIGIT
    PRINT_HUNDREDTHOUSANDS = 255    '/ Blank the HUNDREDTHOUSANDS DIGIT
    PRINT_THOUSANDTHOUSANDS = 255   '/ Blank the THOUSANDTHOUSANDS DIGIT
    PRINT_TENTHDTHOUSANDS = 255     '/ Blank the TENTHDTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 7
    PRINT_ONES = 255                '/ Blank the ONES DIGIT
    PRINT_TENS = 255                '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255            '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255           '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 255        '/ Blank the TENTHOUSANDS DIGIT
    PRINT_HUNDREDTHOUSANDS = 255    '/ Blank the HUNDREDTHOUSANDS DIGIT
    PRINT_THOUSANDTHOUSANDS = 255   '/ Blank the THOUSANDTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 6
    PRINT_ONES = 255                '/ Blank the ONES DIGIT
    PRINT_TENS = 255                '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255            '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255           '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 255        '/ Blank the TENTHOUSANDS DIGIT
    PRINT_HUNDREDTHOUSANDS = 255    '/ Blank the HUNDREDTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 5
    PRINT_ONES = 255            '/ Blank the ONES DIGIT
    PRINT_TENS = 255            '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255        '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255       '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 255    '/ Blank the TENTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 4
    PRINT_ONES = 255        '/ Blank the ONES DIGIT
    PRINT_TENS = 255        '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255    '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255   '/ Blank the THOUSANDS DIGIT
    $endif
    $if NumberDigits = 3
    PRINT_ONES = 255        '/ Blank the ONES DIGIT
    PRINT_TENS = 255        '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255    '/ Blank the HUNDREDS DIGIT
    $endif
    $if NumberDigits = 2
    PRINT_ONES = 255        '/ Blank the ONES DIGIT
    PRINT_TENS = 255        '/ Blank the TENS DIGIT
    $endif    

    ' Configure the interrupts to start.
    INTCONBits_TMR0IF = 0   '/ Clear the Timer0 interrupt flag to begin.
    INTCONBits_TMR0IE = 1   '/ Enable the Timer0 interrupts.
	INTCONBits_GIE = 1	    '/ Enables all High priority interrupts
    ' The Timer0 is running...
    Return
$endif
'---------------------------------------------------------------------------------------
$ifdef ScanTimer2
'/ Subroutine: Init the displays
$define DisplayInit() GoSub DisplayInit_Sub

DisplayInit_Sub:
'/ CONFIGURATION OF TIMER2
'/ Set TMR2 to interrupt on rollover
'/ The TMR2 counter is not loaded.
'/
    #ifdef watchdog_req
    Clrwdt
    #endif
    WDTCON = 0          '/ Without watchdog
    
'/ Same structure for 18F2xk22 & 18F2xk20 & 18F452 series.
    T2CON = 0
    INTCON = 0                                              '/ Clear all interrupts
    While INTCONBits_GIE = 1 : INTCONBits_GIE = 0 : Wend	'/ Make sure global interrupts are off
    PIE1Bits_TMR2IE = 0         '/ Disable the Timer2 interrupts.

    INTCON2Bits_RBPU = 1        '/ All PORTB pull-ups are disabled.
    RCONBits_IPEN = 1           '/ Enable priority levels on interrupts.
    IPR1Bits_TMR2IP = 1         '/ Timer2 for High Interrupt.

    $if _xtal = 4
        ' TMR2 8Bits counter, Postscaler 1/10, Prescaler 1/4, Clock Xtal/4 (Xtal: Xtal 4 Mhz)
        ' For Timer2 interrupt every 10,0 mS 
        T2CON = %01001001
        PR2 = 248
    $endif

    $if NumberDigits = 8 Or NumberDigits = 7
        $if _xtal = 8
            ' TMR2 8Bits counter, Postscaler 1/3, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 8 Mhz)
            ' For Timer2 interrupt every 6,0 mS 
            T2CON = %00010011
            PR2 = 249
        $endif
    $else
        $if _xtal = 8
            ' TMR2 8Bits counter, Postscaler 1/5, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 8 Mhz)
            ' For Timer2 interrupt every 8,0 mS 
            T2CON = %00100011
            PR2 = 249
        $endif
    $endif
    $if _xtal = 16
        ' TMR2 8Bits counter, Postscaler 1/16, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 4 + PLL = 16 Mhz) or (Xtal: Xtal 16Mhz)
        ' For Timer2 interrupt every 10,0 mS 
        T2CON = %01111011
        PR2 = 155        
    $endif
    $if _xtal = 20
        ' TMR2 8Bits counter, Postscaler 1/13, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 20 Mhz)
        ' For Timer2 interrupt every 10,00 mS 
        T2CON = %01100011
        PR2 = 239
    $endif
    $if _xtal = 32
        ' TMR2 8Bits counter, Postscaler 1/4, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 20 Mhz)
        ' For Timer2 interrupt every 8,2 mS 
        T2CON = %01111011
        PR2 = 255
    $endif
    $if _xtal = 40
        ' TMR2 8Bits counter, Postscaler 1/16, Prescaler 1/16, Clock Xtal/4 (Xtal: Xtal 20 Mhz) or (Xtal: Xtal 10 Mhz + PLL)
        ' For Timer2 interrupt every 6,6 mS 
        T2CON = %01111011
        PR2 = 255
    $endif
    $if _xtal = 64
        ' TMR2 8Bits counter, Postscaler 1/16, Prescaler 1/16, Osc Off, Clock Xtal/4 (Xtal: Xtal 16 + PLL = 64Mhz)
        ' For Timer2 interrupt every 4,2 mS
        T2CON = %01111011
        PR2 = 255
    $endif

    ' Updated for Xtal = 80 Mhz, but for testing only because the Timing is only 3,2 mS.
    ' For K20 series.
    $if _xtal = 80
        ' TMR2 8Bits counter, Postscaler 1/16, Prescaler 1/16, Osc Off, Clock Xtal/4 (Xtal: Xtal 20 + PLL = 80Mhz)
        ' For Timer2 interrupt every 3,3 mS
        T2CON = %01111011
        PR2 = 255
    $endif       
       
    TMR2 = 0                    '/ Clear the Timer2 counter.
    Nop    
    T2CONBits_TMR2ON = 1        '/ Start the Timer2.
    '---------------------------------------------------------------------------
    '/ Set the PORTS & Pins to digital.
    ADCON1 = 7
	Output SEGMENTS             '/ Put the display PORT as Output.
	Input PORT_Digits           '/ Put the digit PORT as inputs.

    ' Write the lines OFF at reset depending the Display Mode.
    $if DisplayMode = 0
        $if NumberDigits = 8
        Low DIGIT1
        Low DIGIT2
        Low DIGIT3
        Low DIGIT4
        Low DIGIT5          
        Low DIGIT6 
        Low DIGIT7
        Low DIGIT8
        $endif
        $if NumberDigits = 7
        Low DIGIT1
        Low DIGIT2
        Low DIGIT3
        Low DIGIT4
        Low DIGIT5          
        Low DIGIT6 
        Low DIGIT7
        $endif
        $if NumberDigits = 6
        Low DIGIT1
        Low DIGIT2
        Low DIGIT3
        Low DIGIT4
        Low DIGIT5          
        Low DIGIT6 
        $endif
        $if NumberDigits = 5
        Low DIGIT1
        Low DIGIT2
        Low DIGIT3
        Low DIGIT4
        Low DIGIT5          
        $endif
        $if NumberDigits = 4
        Low DIGIT1
        Low DIGIT2
        Low DIGIT3
        Low DIGIT4  
        $endif
        $if NumberDigits = 3
        Low DIGIT1
        Low DIGIT2
        Low DIGIT3
        $endif
        $if NumberDigits = 2
        Low DIGIT1
        Low DIGIT2
        $endif                
    $else
        $if NumberDigits = 8
        High DIGIT1
        High DIGIT2
        High DIGIT3
        High DIGIT4
        High DIGIT5          
        High DIGIT6 
        High DIGIT7
        High DIGIT8
        $endif
        $if NumberDigits = 7
        High DIGIT1
        High DIGIT2
        High DIGIT3
        High DIGIT4
        High DIGIT5          
        High DIGIT6 
        High DIGIT7
        $endif
        $if NumberDigits = 6
        High DIGIT1
        High DIGIT2
        High DIGIT3
        High DIGIT4
        High DIGIT5          
        High DIGIT6 
        $endif
        $if NumberDigits = 5
        High DIGIT1
        High DIGIT2
        High DIGIT3
        High DIGIT4
        High DIGIT5          
        $endif
        $if NumberDigits = 4
        High DIGIT1
        High DIGIT2
        High DIGIT3
        High DIGIT4  
        $endif
        $if NumberDigits = 3
        High DIGIT1
        High DIGIT2
        High DIGIT3
        $endif
        $if NumberDigits = 2
        High DIGIT1
        High DIGIT2
        $endif        
    $endif

    ' Blanks directly the Port at reset depending of the option.
    $if DisplayMode = 0
    SEGMENTS = 0                '/ All displays cleared for common cathode.
    $else
    SEGMENTS = 255              '/ All displays cleared for common anode.
    $endif

	TIMESHARE = 1		        '/ Reset the time share variable

    '/ Blanks the Displays at reset.
    $if NumberDigits = 8
    PRINT_ONES = 255                '/ Blank the ONES DIGIT
    PRINT_TENS = 255                '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255            '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255           '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 255        '/ Blank the TENTHOUSANDS DIGIT
    PRINT_HUNDREDTHOUSANDS = 255    '/ Blank the HUNDREDTHOUSANDS DIGIT
    PRINT_THOUSANDTHOUSANDS = 255    '/ Blank the THOUSANDTHOUSANDS DIGIT
    PRINT_TENTHDTHOUSANDS = 255    '/ Blank the TENTHDTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 7
    PRINT_ONES = 255                '/ Blank the ONES DIGIT
    PRINT_TENS = 255                '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255            '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255           '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 255        '/ Blank the TENTHOUSANDS DIGIT
    PRINT_HUNDREDTHOUSANDS = 255    '/ Blank the HUNDREDTHOUSANDS DIGIT
    PRINT_THOUSANDTHOUSANDS = 255    '/ Blank the THOUSANDTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 6
    PRINT_ONES = 255                '/ Blank the ONES DIGIT
    PRINT_TENS = 255                '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255            '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255           '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 255        '/ Blank the TENTHOUSANDS DIGIT
    PRINT_HUNDREDTHOUSANDS = 255    '/ Blank the TENTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 5
    PRINT_ONES = 255            '/ Blank the ONES DIGIT
    PRINT_TENS = 255            '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255        '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255       '/ Blank the THOUSANDS DIGIT
    PRINT_TENTHOUSANDS = 255    '/ Blank the TENTHOUSANDS DIGIT
    $endif
    $if NumberDigits = 4
    PRINT_ONES = 255        '/ Blank the ONES DIGIT
    PRINT_TENS = 255        '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255    '/ Blank the HUNDREDS DIGIT
    PRINT_THOUSANDS = 255   '/ Blank the THOUSANDS DIGIT
    $endif
    $if NumberDigits = 3
    PRINT_ONES = 255        '/ Blank the ONES DIGIT
    PRINT_TENS = 255        '/ Blank the TENS DIGIT
    PRINT_HUNDREDS = 255    '/ Blank the HUNDREDS DIGIT
    $endif
    $if NumberDigits = 2
    PRINT_ONES = 255        '/ Blank the ONES DIGIT
    PRINT_TENS = 255        '/ Blank the TENS DIGIT
    $endif    

    ' Configure the interrupts to start.
    PIR1Bits_TMR2IF = 0     '/ Clear the Timer0 interrupt flag to begin.
    PIE1Bits_TMR2IE = 1     '/ Enable the Timer0 interrupts.
	INTCONBits_GIE = 1	    '/ Enables all High priority interrupts
    ' The Timer2 is running...
    Return
$endif
'/==============================================================================
'/ Shifting Macros:
'/ The Original data is in DISP_ARRAY.
'/ The new data shifted will be in the DISP_ARRAYcop.
'/ The FSR0 pointer is used for the DISP_ARRAY.
'/ The FSR1 pointer is used for the DISP_ARRAYcop. 
'/
'/ First parameter: Number of display shifts.
'/ Second parameter: Value of delay in mS between shift.
'/
$if ShiftingDigits = YES

#ifMacro- _mRotateLeftDispArray, _mRotateRightDispArray 
    Dim RotateDelay As Word System
    Dim DispLenArray As Byte System
#endIfMacro-

$define RotateRightDisplay(pNumberShift, pDelayMS) '
    $if NumberDigits < 4                           '
    $error "Too few digits for shifting"           '
    $endif                                         '
    $if pNumberShift = 0                           '
    $error "Too few parameter [Number of shifts]"  '
    $endif                                         '
    $if pDelayMS = 0                               '
    $error "Too few parameter [Number of MS]"      '
    $endif                                         '
    PRODL = pNumberShift                           '
    RotateDelay = pDelayMS                         '
    _mRotateRightDispArray

_mRotateRightDispArray Macro-
    GoSub RotateRightDispArray_Sub
Endm

#ifMacro- _mRotateRightDispArray
RotateRightDispArray_Sub:
    Repeat 
        #ifdef watchdog_req
        Clrwdt
        #endif
        GoSub RotateRightDigit_Sub   '/ Shift the array to the Right.
        DelayMS RotateDelay         '/ Delay between rotations.
        Dec PRODL
    Until PRODL = 0                 '/ Shift all the digits.
    Return

RotateRightDigit_Sub:
'    DispLenArray = Len (DISP_ARRAY) '/ Accordinly with the number of digits.
    DispLenArray = LenghtDISP_ARRAY
    FSR0 = VarPtr (DISP_ARRAY)      '/ Start address of the array
    FSR1 = VarPtr (DISP_ARRAYcop)   '/ Start address of the array
    PRODH = INDF0                   '/ Save the first original byte.
    Inc FSR0                        '/ Shift pointer in origen + 1.
    Repeat
        #ifdef watchdog_req
        Clrwdt
        #endif
        POSTINC1 = POSTINC0         '/ Copy all bytes
        Dec DispLenArray         
    Until DispLenArray = 0
    Dec FSR1
    INDF1 = PRODH                   '/ Place the first byte            
    DisPlayInterruptEnable = 0 
    DISP_ARRAY = DISP_ARRAYcop      '/ Return the array to print.
    DisPlayInterruptEnable = 1 
    Return
#endIfMacro-    
'/------------------------------------------------------------------------------
$define RotateLeftDisplay(pNumberShift, pDelayMS) '
    $if NumberDigits < 4                          '
    $error "Too few digits for shifting"          '
    $endif                                        '
    $if pNumberShift = 0                          '
    $error "Too few parameter [Number of shifts]" '
    $endif                                        '
    $if pDelayMS = 0                              '
    $error "Too few parameter [Number of MS]"     '
    $endif                                        '
    PRODL = pNumberShift                          '
    RotateDelay = pDelayMS                        '
    _mRotateLeftDispArray

_mRotateLeftDispArray Macro-
    GoSub RotateLeftDispArray_Sub
Endm

#ifMacro- _mRotateLeftDispArray
RotateLeftDispArray_Sub:
    Repeat 
        #ifdef watchdog_req
        Clrwdt
        #endif
        GoSub RotateLeftDigit_Sub    '/ Shift the array to the Left.
        DelayMS RotateDelay         '/ Delay between rotations.
        Dec PRODL
    Until PRODL = 0                 '/ Shift all the digits.
    Return

RotateLeftDigit_Sub:
'    DispLenArray = Len (DISP_ARRAY) '/ Accordinly with the number of digits.
    DispLenArray = LenghtDISP_ARRAY
    FSR0 = VarPtr (DISP_ARRAY)      '/ Start address of the array
    FSR1 = VarPtr (DISP_ARRAYcop)   '/ Start address of the array
    FSR0 = FSR0 + DispLenArray      '/ End of address of the array
    FSR1 = FSR1 + DispLenArray      '/ End of address of the array
    Dec FSR0                        '/ Place the pointer at the last position of the array.
    Dec FSR1                        '/ Place the pointer At the last position of the array.
    PRODH = INDF0                   '/ Save the last original byte.
    Dec FSR0                        '/ Shift pointer to final - 1.
    Repeat
        #ifdef watchdog_req
        Clrwdt
        #endif
        POSTDEC1 = POSTDEC0         '/ Copy all bytes
        Dec DispLenArray         
    Until DispLenArray = 0
    Inc FSR1
    INDF1 = PRODH                   '/ Place the last byte
    DisPlayInterruptEnable = 0 
    DISP_ARRAY = DISP_ARRAYcop
    DisPlayInterruptEnable = 1      '/ Return the array To Print.
    Return
#endIfMacro-    
$endif
'===============================================================================
_MultiplexDigitsExit:
'===============================================================================
    DisplayInit()       '/ Initialize the Displays.
'===============================================================================
$endif  ' _MultiplexDigits
'===============================================================================

    
